--[[
			(True) Adaptive User Interface
			
			To the extent possible under law, Matthew D. Lee has
			waived all copyright and related or neighboring rights to the 
			(True) Adaptive User Interface.  This work is published from the
			United States of America and is considered Public Domain.
			
			Share and share a like.
]]

--[[
			Credit to Baine @ http://www.wowinterface.com/forums/member.php?userid=150634
			for creating helper functions and making me realize I should localize all of
			the class abilities, otherwise it would fail for anything other than US English
			(or EU English) localization.  And protentially for a lot more things to come.
]]

--[[
			Helper Functions
]]

-- To Determine whether a glyph is used or not.
local function testbool(a, condition)
	if (condition == "yes" or not condition) then
		if (a) then
			return 0
		else
			return nil
		end
	else
		if (not a) then
			return 0
		else
			return nil
		end
	end
end

Glyph = function(condition)
	local present = false
	for i = 1, GetNumGlyphSockets() do
		local enabled, glypType, glyphTooltipIndex, glyphSpellID = GetGlyphSocketInfo(i)
		if (glyphSpellID == condition[1]) then
			present = true
			break
		end
	end
	return testbool(present, condition[2])
end

-- To get a Localized SpellName
local GetSpellName = function(spellId)
	return GetSpellInfo(spellId)
end

--[[
			Important spells or abilities that may effect many different rotations.
			Will add more if and when applicable.
]]

locale = {};

locale.basic = {
	["hr"] = GetSpellName(32182),			-- Heroism
	["bl"] = GetSpellName(2825),			-- Bloodlust
	["tw"] = GetSpellName(80353),			-- Time Warp
};

--[[
			Hunter Localization & Priorities
]]

locale.hunter = {
	["gl"] = GetSpellName(2649),			-- Growl
	["srs"] = GetSpellName(1978),			-- Serpent Sting
	["kc"] = GetSpellName(34026),			-- Kill Command
	["ks"] = GetSpellName(53351),			-- Kill Shot
	["as"] = GetSpellName(3044),			-- Arcane Shot
	["ss"] = GetSpellName(56641),			-- Steady Shot
	["ai"] = GetSpellName(19434),			-- Aimed Shot
	["cos"] = GetSpellName(77767),			-- Cobra Shot
	["es"] = GetSpellName(53301),			-- Explosive Shot
	["bkar"] = GetSpellName(3674),			-- Black Arrow
	["cs"] = GetSpellName(53209),			-- Chimera Shot
	["rf"] = GetSpellName(3045),			-- Rapid Fire
	["fr"] = GetSpellName(82926),			-- Fire! (Buff)
}


-- Hunter 4.3 Priority for CLCinfo (modified for OpenRDX) by Phanx @ http://www.wowinterface.com/forums/showpost.php?p=251728&postcount=2
function TRUESYS.PriorityHunterBeastMastery()
	local gcd = 1.5
	local level = UnitLevel("player")
	local focus = UnitPower("player")
	local duration, _

	-- 1. If your pet is on the target, Kill Command.
	if UnitExists("pet") and UnitHealth("pet") > 1 and IsSpellInRange(RDXSS.GetSpellBookId(locale.hunter["gl"], BOOKTYPE_PET),"pet","pettarget") == 1 and focus > 50 then
		_, duration = GetSpellCooldown(locale.hunter["kc"])
		if duration <= gcd then
			return GetSpellTexture(locale.hunter["kc"])
		end
	end

	-- 2. If the mob is going to live long enough, apply Serpent Sting.
	if focus > 25 then
		if not UnitDebuff("target", locale.hunter["srs"]) then
			_, duration = GetSpellCooldown(locale.hunter["srs"])
			if duration <= gcd then
				return GetSpellTexture(locale.hunter["srs"])
			end
		end
	end

	-- 3. If mob is below 20% health, Kill Shot.
	_, duration = GetSpellCooldown(locale.hunter["ks"])
	if duration <= gcd and ( UnitHealth("target") / UnitHealthMax("target") < 0.2 ) then
		return GetSpellTexture(locale.hunter["ks"])
	end

	-- 4. If you will have enough focus to use Kill Command on cooldown, use Arcane Shot
	if focus > 85 then
		return GetSpellTexture(locale.hunter["as"])
	end

	-- 5. Cobra Shot or Steady Shot.
	if level < 81 then
		return GetSpellTexture(locale.hunter["ss"])
	else
		return GetSpellTexture(locale.hunter["cos"])
	end
end

-- Hunter 4.3 Priority for CLCinfo (modified for OpenRDX) by Phanx @ http://www.wowinterface.com/forums/showpost.php?p=251728&postcount=2
function TRUESYS.PriorityHunterSurvival()
	local gcd = 1.5
	local level = UnitLevel("player")
	local focus = UnitPower("player")
	local duration, _

	if not UnitCanAttack("player", "target") then
		return
	end

	-- 1. If the mob is going to live long enough, apply Serpent Sting.
	if focus > 25 then
		if not UnitDebuff("target", locale.hunter["srs"]) then
			_, duration = GetSpellCooldown(locale.hunter["srs"])
			if duration <= gcd then
				return GetSpellTexture(locale.hunter["srs"])
			end
		end
	end
	
	-- 2. Explosive Shot
	if focus > 50 then
		_, duration = GetSpellCooldown(locale.hunter["es"])
		if duration <= gcd then
			return GetSpellTexture(locale.hunter["es"])
		end
	end

	-- 3. If mob is below 20% health, Kill Shot.
	_, duration = GetSpellCooldown(locale.hunter["ks"])
	if duration <= gcd and ( UnitHealth("target") / UnitHealthMax("target") < 0.2 ) then
		return GetSpellTexture(locale.hunter["ks"])
	end

	-- 4. If mob will be alive long enough, Black Arrow.
	if level < 61 and focus > 35 then
		_, duration = GetSpellCooldown(locale.hunter["bkar"])
		if duration <= gcd and ( UnitHealth("target") / dps > 15 ) then
			return GetSpellTexture(locale.hunter["bkar"])
		end
	end

	-- 5. If you will have enough focus to use Explosive Shot and Black Arrow, use Arcane Shot.
	if focus > 85 then
		return GetSpellTexture(locale.hunter["as"])
	end

	-- 6. Cobra Shot or Steady Shot.
	if level < 81 then
		return GetSpellTexture(locale.hunter["ss"])
	else
		return GetSpellTexture(locale.hunter["cos"])
	end
end

-- Hunter 4.3 Priority based off Ovale (modified for OpenRDX) by Baine @ https://gist.github.com/1845551
function TRUESYS.PriorityHunterMarksmanship()
	local gcd = 1.5
	-- local level = UnitLevel("player")		Add this later.
	local focus = UnitPower("player")
	local duration, _

	if focus > 24 and ( UnitHealth("target") / UnitHealthMax("target") < 0.9 ) then
		if not UnitDebuff("target", locale.hunter["srs"]) then
			_, duration = GetSpellCooldown(locale.hunter["srs"])
			if duration <= gcd then
				return GetSpellTexture(locale.hunter["srs"])
			end
		end
	end

	if focus > 49 and( UnitHealth("target") / UnitHealthMax("target") > 0.9 ) then
		_, duration = GetSpellCooldown(locale.hunter["ai"])
		if duration <= gcd then
			return GetSpellTexture(locale.hunter["ai"])
		end
	end


	_, duration = GetSpellCooldown(locale.hunter["ks"])
	if duration <= gcd and ( UnitHealth("target") / UnitHealthMax("target") < 0.2 ) then
		return GetSpellTexture(locale.hunter["ks"])
	end

	if UnitBuff("player", locale.hunter["fr"]) then
		return GetSpellTexture(locale.hunter["ai"])
	end

	if Glyph("56841") then
		if ( UnitHealth("target") / UnitHealthMax("target") > 0.9 ) or UnitBuff("player", locale.hunter["rf"]) or UnitBuff("player", locale.basic["hr"]) or UnitBuff("player", locale.basic["bl"]) or UnitBuff("player", locale.basic["tw"]) then
			if focus > 49 then
				return GetSpellTexture(locale.hunter["ai"])
			end
		end
		if focus > 24 then
			return GetSpellTexture(locale.hunter["as"])
		end
	else
		if focus > 79 or ( UnitHealth("target") / UnitHealthMax("target") > 0.9 ) or UnitBuff("player", L[locale.hunter["rf"]]) or UnitBuff("player", locale.basic["hr"]) or UnitBuff("player", locale.basic["bl"]) or UnitBuff("player", locale.basic["tw"]) then
			if focus > 49 then
				return GetSpellTexture(locale.hunter["ai"])
			end
		end
	end

	return GetSpellTexture(locale.hunter["ss"])
end

--[[
			Warrior Localization & Priorities
]]

locale.warrior = {
	["bshout"] = GetSpellName(6673),		--BattleShout
	["exbuff"] = GetSpellName(90806),		--Executioner Buff
	["ex"] = GetSpellName(5308),			--Execute
	["cs"] = GetSpellName(86346),			--Colossus Smash
	["bt"] = GetSpellName(23881),			--Blood Thirst
	["er"] = GetSpellName(14202),			--Enrage
	["dw"] = GetSpellName(12292),			--Death Wish
	["rb"] = GetSpellName(85288),			--Raging Blow
	["br"] = GetSpellName(18499),			--Berserker Rage
	["bs"] = GetSpellName(46916),			--Blood Surge buff
	["sl"] = GetSpellName(1464),			--Slam
	["vcbuff"] = GetSpellName(32216),		--Victorious Buff
	["vr"] = GetSpellName(34428),			--Victory Rush
}

-- Warrior 4.3 Priority for OpenRDX by Baine @ https://github.com/Baine/TrueAui/commit/e14e6454190bd3f0d473f66a2116afcc16491f9d
function TRUESYS.PriorityWarriorFury()
	local gcd = 1.5
	local rage = UnitPower("player")
	local duration, _

	--rage below 20, return battleshout
	if rage < 20 then
	  return GetSpellTexture(locale.warrior["bshout"])
	end
	_, _, _, excount, _, _, exptime = UnitBuff("player", locale.warrior["exbuff"])
	if(excount==nil) then excount = 0 end
	if(exptime==nil) then exptime = 0 end

	if ( UnitHealth("target") / UnitHealthMax("target") < 0.2 and exptime < 2)  then
	  return GetSpellTexture(locale.warrior["ex"])
	end

	_, duration = GetSpellCooldown(locale.warrior["cs"])
	if duration <= gcd then
	  return GetSpellTexture(locale.warrior["cs"])
	end

	_, _, _, excount, _, _, exptime = UnitBuff("player", locale.warrior["exbuff"])
	if(excount==nil) then excount = 0 end
	if(exptime==nil) then exptime = 0 end
	if(UnitHealth("target") / UnitHealthMax("target") < 0.2 and excount < 5) then
		return GetSpellTexture(locale.warrior["ex"])
	end

	_, duration = GetSpellCooldown(locale.warrior["bt"])
	if duration < gcd then
		return GetSpellTexture(locale.warrior["bt"])
	end

	local _, _, _, _, hasTG, _= GetTalentInfo(2,20);
	local _, rbCD = GetSpellCooldown(locale.warrior["rb"])
	if hasTG == 1 then
		if(rage > 15 and not(UnitBuff("player",locale.warrior["er"])  or UnitBuff("player", locale.warrior["dw"])) and rbCD < gcd) then
			return GetSpellTexture(locale.warrior["br"])
		end

		if(UnitBuff("player",locale.warrior["er"])  or UnitBuff("player", locale.warrior["dw"]) and rbCD < gcd) then
			return GetSpellTexture(locale.warrior["rb"])
		end
	end

	if UnitBuff("player", locale.warrior["bs"]) then
		return GetSpellTexture(locale.warrior["sl"])
	end

	if rage > 49 and ( UnitHealth("target") / UnitHealthMax("target") < 0.2 ) then
		return GetSpellTexture(locale.warrior["ex"])
	end

	if hasTG == 0 then
		if(rage > 15 and not(UnitBuff("player",locale.warrior["er"])  or UnitBuff("player", locale.warrior["dw"])) and rbCD < gcd) then
			return GetSpellTexture(locale.warrior["br"])
		end

		if(UnitBuff("player",locale.warrior["er"])  or UnitBuff("player", locale.warrior["dw"]) and rbCD < gcd) then
			return GetSpellTexture(locale.warrior["rb"])
		end
	end

	if(UnitBuff("player", locale.warrior["vcbuff"])) then
		return GetSpellTexture(locale.warrior["vr"])
	end

	if rage < 70 then
		return GetSpellTexture(locale.warrior["bs"])
	end

	return nil

end

--[[
			Function to iterate over class/spec, returns the icon generated by the respective rotation function.
			Credit to Baine @ https://github.com/Baine/TrueAui/commit/c604af6685e0e2cb930d2e13ade869a2ab45435a
]]

function TRUESYS.RotationByClass()
	local class = select(2, UnitClass("player"))
	local mastery = GetPrimaryTalentTree()
	
	if class == "DEATHKNIGHT" then
		return nil
	end
	
	if class == "DRUID" then
		return nil
	end
	
	if class == "HUNTER" then
		if mastery == 1 then
			return TRUESYS.PriorityHunterBeastMastery()
		end
		
		if mastery == 2 then
			return TRUESYS.PriorityHunterMarksmanship()
		end
		
		if mastery == 3 then
			return TRUESYS.PriorityHunterSurvival()
		end
		
		return nil
	end
	
	if class == "MAGE" then
		return nil
	end
	
	if class == "PALADIN" then
		return nil
	end
	
	if class == "PRIEST" then
		return nil
	end
	
	if class == "ROGUE" then
		return nil
	end
	
	if class == "SHAMAN" then
		return nil
	end
	
	if class == "WARLOCK" then
		return nil
	end
	
	if class == "WARRIOR" then
		if mastery == 2 then
			return TRUESYS.PriorityWarriorFury()
		end
		return nil
	end
	

end