--[[
			(True) Adaptive User Interface
			
			To the extent possible under law, Matthew D. Lee has
			waived all copyright and related or neighboring rights to the 
			(True) Adaptive User Interface.  This work is published from the
			United States of America and is considered Public Domain.
			
			Share and share a like.
]]

--[[
			Define a few variables to help with automation, customization, and package protection.
			
			Thanks to Saiket, Seerah, & SDPhantom @ http://www.wowinterface.com/forums/showthread.php?p=252068
			for helping me hook RDX.IsProtectedPkg, allowing me to protect my package names.
			
			The reason for this is, I'd prefer that my work be kept intact and updated in a
			consistent manner.  This promotes people to copy my work using the copy package tool 
			and make derivatives in their own, unique new package - which is ultimately what I
			want, people to learn off of this one and use it to make their own, while allowing me to
			directly overwrite my own packages every update!
]]

TRUESYS.PKG = {
	"true_core",
	"true_unitframes",
	"true_multiframes"
};

TRUESYS.MD = {											-- Package Metadata
	TRUESYS.GetVersion(),								-- infoVersion
	"Unlimit",											-- infoAuthor
	"Demon Soul US",									-- infoRealm
	"matt@wowrdx.com",									-- infoAuthorEmail
	"manversuspixel.com",								-- infoAuthorWebSite
	"TrueAui",											-- infoComment
	"infoIsShare",
	"infoIsImmutable",
	"infoIsIndelible"
};

local truePkg = {};
truePkg[TRUESYS.PKG[1]] = true;
truePkg[TRUESYS.PKG[2]] = true;
truePkg[TRUESYS.PKG[3]] = true;

local original_IsProtectedPkg = RDXDB.IsProtectedPkg;
function RDXDB.IsProtectedPkg(name)
	local val = original_IsProtectedPkg(name);
	if val ~= nil then
		return val;
	else
		return truePkg[name];
	end
end

--[[
			Create (True) Adaptive User Interface Packages & Objects
			
			Thanks to Seerah and Xrystal for showing me what a loop is
			@ http://www.wowinterface.com/forums/showthread.php?p=252602
]]

RDXEvents:Bind("INIT_DATABASE_LOADED", nil, function()
	local version = RDXDB.GetPackageMetadata(TRUESYS.PKG[1], "infoVersion")
	local update

	if not RDXDB.GetPackage(TRUESYS.PKG[1]) then return nil; end
	if not version or version < TRUESYS.MD[1] then update = true; end

	if update == true then
		RDX.print("|cff89F559(True)|r |c007ffcffAdaptive User Interface|r updated to the latest version!");
		for i=1,3 do
			RDXDB._EmptyPackage(TRUESYS.PKG[i], true)
		end
	end
end);

RDXEvents:Bind("INIT_DATABASE_LOADED", nil, function()
	-- Define a local to package name and set package metadata
	local core = RDXDB.GetOrCreatePackage(TRUESYS.PKG[1]);
	local unitframes = RDXDB.GetOrCreatePackage(TRUESYS.PKG[2]);
	local multiframes = RDXDB.GetOrCreatePackage(TRUESYS.PKG[3]);
	for i=1,3 do
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoVersion", TRUESYS.MD[1]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoAuthor", TRUESYS.MD[2]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoAuthorRealm", TRUESYS.MD[3]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoAuthorEmail", TRUESYS.MD[4]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoAuthorWebSite", TRUESYS.MD[5]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoComment", TRUESYS.MD[6]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoIsShare", TRUESYS.MD[7]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoIsImmutable", TRUESYS.MD[8]);
		RDXDB.SetPackageMetadata(TRUESYS.PKG[i], "infoIsIndelible", TRUESYS.MD[9]);
	end

	-- true_core Package
		-- Desktop Objects
	if not core["True"] then
		core["True"] = {
			["ty"] = "AUI",
			["version"] = 1,
			["data"] = {
				["solo2"] = "true_core:true_desktop_default",
				["raidflag"] = 1,
				["arenaflag2"] = 1,
				["party"] = "true_core:true_desktop_party",
				["pvp2"] = "true_core:true_desktop_raid",
				["raid"] = "true_core:true_desktop_raid",
				["raid2"] = "true_core:true_desktop_raid",
				["arenaflag"] = 1,
				["partyflag"] = 1,
				["raidflag2"] = 1,
				["soloflag"] = 1,
				["arena2"] = "true_core:true_desktop_default",
				["partyflag2"] = 1,
				["party2"] = "true_core:true_desktop_party",
				["soloflag2"] = 1,
				["pvpflag"] = 1,
				["solo"] = "true_core:true_desktop_default",
				["arena"] = "true_core:true_desktop_default",
				["pvpflag2"] = 1,
				["pvp"] = "true_core:true_desktop_raid",
			},
		};
	end
	if not core["true_desktop_default"] then
		core["true_desktop_default"] = {
			["ty"] = "Desktop",
			["version"] = 2,
			["data"] = {
				{
					["strata"] = "LOW",
					["b"] = 0,
					["scale"] = 1,
					["dgp"] = true,
					["feature"] = "Desktop main",
					["offsettop"] = 0,
					["bkd"] = {
						["bgFile"] = "Interface\\Addons\\VFL\\Skin\\a80black",
						["tileSize"] = 16,
						["tile"] = true,
						["edgeFile"] = "Interface\\Addons\\VFL\\Skin\\HalBorder",
						["edgeSize"] = 8,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
					["l"] = 0,
					["offsetleft"] = 0,
					["anchorxrid"] = -200,
					["r"] = 1228.800048828125,
					["offsetbottom"] = 0,
					["alpha"] = 1,
					["dock"] = {
						["BOTTOM"] = {
							["id"] = "true_core:actionbar_window",
							["x"] = 0,
							["point"] = "BOTTOM",
							["y"] = -35,
						},
						["RIGHT"] = {
							["id"] = "true_core:sidebar_window",
							["x"] = 25,
							["point"] = "RIGHT",
							["y"] = 0,
						},
						["BOTTOMRIGHT"] = {
							["id"] = "true_core:minimap_window",
							["x"] = 25,
							["point"] = "BOTTOMRIGHT",
							["y"] = -25,
						},
						["TOPRIGHT"] = {
							["id"] = "true_core:aura_buff_window",
							["x"] = 25,
							["point"] = "TOPRIGHT",
							["y"] = 25,
						},
						["TOPLEFT"] = {
							["id"] = "true_core:chatframe_window",
							["x"] = -25,
							["point"] = "TOPLEFT",
							["y"] = 25,
						},
					},
					["root"] = true,
					["t"] = 768.0000610351563,
					["open"] = true,
					["anchoryrid"] = -200,
					["anchorx"] = 75.657139627882,
					["offsetright"] = 0,
					["name"] = "root",
					["font"] = {
						["name"] = "Default10",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\VFL\\Fonts\\LiberationSans-Regular.ttf",
						["title"] = "Default 10pt",
						["size"] = 10,
					},
					["tex"] = {
						["color"] = {
							["a"] = 1,
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["path"] = "Interface\\Addons\\RDX\\Skin\\bar1",
						["blendMode"] = "BLEND",
					},
					["title"] = "default",
					["anchory"] = 134.90279833818,
					["ap"] = "TOPLEFT",
				},
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "Actionbar Objects",
				},
				{
					["strata"] = "MEDIUM",
					["r"] = 746.0571348667145,
					["scale"] = 1,
					["t"] = 69.4857191218648,
					["open"] = true,
					["feature"] = "desktop_window",
					["name"] = "true_core:actionbar_window",
					["alpha"] = 1,
					["anchor"] = "TOPLEFT",
					["l"] = 482.74285197258,
					["b"] = 25.59999972581863,
					["dock"] = {
						["BOTTOMLEFT"] = {
							["id"] = "true_core:petbar_window",
							["x"] = 10,
							["point"] = "BOTTOMRIGHT",
							["y"] = 0,
						},
						["TOP"] = {
							["id"] = "true_unitframes:player_window",
							["x"] = 0,
							["point"] = "BOTTOM",
							["y"] = -80,
						},
						["BOTTOM"] = {
							["id"] = "root",
							["x"] = 0,
							["point"] = "BOTTOM",
							["y"] = 35,
						},
					},
				}, 
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["RIGHT"] = {
							["id"] = "root",
							["x"] = -25,
							["point"] = "RIGHT",
							["y"] = 0,
						},
					},
					["scale"] = 1,
					["t"] = 515.6571373343468,
					["open"] = 1,
					["feature"] = "desktop_window",
					["name"] = "true_core:sidebar_window",
					["r"] = 1210.514451320851,
					["b"] = 252.3428767616406,
					["l"] = 1144.685880597317,
					["alpha"] = 1,
					["ap"] = "TOPLEFT",
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["BOTTOM"] = {
							["id"] = "true_unitframes:pet_window",
							["x"] = 0,
							["point"] = "TOP",
							["y"] = 5,
						},
						["BOTTOMRIGHT"] = {
							["id"] = "true_core:actionbar_window",
							["x"] = -10,
							["point"] = "BOTTOMLEFT",
							["y"] = 0,
						},
					},
					["scale"] = 1,
					["t"] = 69.4857191218648,
					["alpha"] = 0.5,
					["feature"] = "desktop_window",
					["name"] = "true_core:petbar_window",
					["r"] = 475.4285216937751,
					["anchor"] = "TOPLEFT",
					["l"] = 365.7142371545524,
					["b"] = 25.59999972581863,
					["open"] = true,
				},
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "Core Design Objects",
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["BOTTOMRIGHT"] = {
							["id"] = "root",
							["x"] = -25,
							["point"] = "BOTTOMRIGHT",
							["y"] = 25,
						},
					},
					["scale"] = 1,
					["t"] = 120.685712993145,
					["open"] = 1,
					["feature"] = "desktop_window",
					["name"] = "true_core:minimap_window",
					["r"] = 1210.514451320851,
					["b"] = 18.28571269478118,
					["l"] = 1108.114452417576,
					["alpha"] = 1,
					["ap"] = "TOPLEFT",
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["TOPLEFT"] = {
							["id"] = "root",
							["x"] = 25,
							["point"] = "TOPLEFT",
							["y"] = -25,
						},
					},
					["scale"] = 1,
					["t"] = 749.7143669704019,
					["open"] = true,
					["feature"] = "desktop_window",
					["name"] = "true_core:chatframe_window",
					["alpha"] = 1,
					["anchor"] = "TOPLEFT",
					["l"] = 18.28571269478118,
					["r"] = 274.2857336694851,
					["b"] = 650.9714662422448,
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["BOTTOMRIGHT"] = {
							["id"] = "true_core:aura_debuff_window",
							["x"] = -0,
							["point"] = "TOPRIGHT",
							["y"] = 20,
						},
						["TOPRIGHT"] = {
							["id"] = "root",
							["x"] = -25,
							["point"] = "TOPRIGHT",
							["y"] = -25,
						},
					},
					["scale"] = 1,
					["t"] = 749.7143669704019,
					["alpha"] = 1,
					["feature"] = "desktop_window",
					["b"] = 705.8286531547128,
					["name"] = "true_core:aura_buff_window",
					["r"] = 1210.514451320851,
					["anchor"] = "TOPLEFT",
					["l"] = 771.6572238782464,
					["open"] = true,
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["TOPRIGHT"] = {
							["id"] = "true_core:aura_buff_window",
							["x"] = 0,
							["point"] = "BOTTOMRIGHT",
							["y"] = -20,
						},
					},
					["scale"] = 1,
					["t"] = 691.2000818828165,
					["alpha"] = 1,
					["feature"] = "desktop_window",
					["b"] = 647.3143234242707,
					["name"] = "true_core:aura_debuff_window",
					["r"] = 1210.514451320851,
					["anchor"] = "TOPLEFT",
					["l"] = 771.6572238782464,
					["open"] = true,
				},
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "Player Objects",
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["RIGHT"] = {
							["id"] = "true_unitframes:focus_window",
							["x"] = -5,
							["point"] = "LEFT",
							["y"] = 0,
						},
						["TOP"] = {
							["id"] = "true_unitframes:target_window",
							["x"] = 0,
							["point"] = "BOTTOM",
							["y"] = -35,
						},
						["BOTTOM"] = {
							["id"] = "true_core:actionbar_window",
							["x"] = 0,
							["point"] = "TOP",
							["y"] = 80,
						},
					},
					["scale"] = 1,
					["t"] = 146.2857015582495,
					["open"] = 1,
					["feature"] = "desktop_window",
					["name"] = "true_unitframes:player_window",
					["r"] = 705.8286085118562,
					["b"] = 127.9999986290932,
					["l"] = 522.971422970295,
					["alpha"] = 1,
					["ap"] = "TOPLEFT",
				}, -- [10]
				{
					["strata"] = "MEDIUM",
					["r"] = 819.1999912261963,
					["scale"] = 1,
					["t"] = 146.2857015582495,
					["open"] = true,
					["feature"] = "desktop_window",
					["name"] = "true_unitframes:focus_window",
					["alpha"] = 1,
					["anchor"] = "TOPLEFT",
					["l"] = 709.4857513298302,
					["b"] = 127.9999986290932,
					["dock"] = {
						["LEFT"] = {
							["id"] = "true_unitframes:player_window",
							["x"] = 5,
							["point"] = "RIGHT",
							["y"] = 0,
						},
					},
				}, -- [11]
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["TOP"] = {
							["id"] = "true_core:petbar_window",
							["x"] = 0,
							["point"] = "BOTTOM",
							["y"] = -5,
						},
					},
					["scale"] = 1,
					["t"] = 21.94285830293381,
					["alpha"] = 1,
					["feature"] = "desktop_window",
					["name"] = "true_unitframes:pet_window",
					["r"] = 475.4285663366318,
					["anchor"] = "TOPLEFT",
					["l"] = 365.7142371545524,
					["b"] = 3.657143864291044,
					["open"] = true,
				},
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "Target Objects",
				},
				{
					["strata"] = "MEDIUM",
					["dock"] = {
						["RIGHT"] = {
							["id"] = "true_unitframes:targettarget_window",
							["x"] = -5,
							["point"] = "LEFT",
							["y"] = 0,
						},
						["BOTTOM"] = {
							["id"] = "true_unitframes:player_window",
							["x"] = 0,
							["point"] = "TOP",
							["y"] = 35,
						},
					},
					["scale"] = 1,
					["t"] = 190.1714265346527,
					["open"] = true,
					["feature"] = "desktop_window",
					["name"] = "true_unitframes:target_window",
					["alpha"] = 1,
					["anchor"] = "TOPLEFT",
					["l"] = 522.971422970295,
					["r"] = 705.8286085118562,
					["b"] = 171.8857124447823,
				},
				{
					["strata"] = "MEDIUM",
					["r"] = 819.1999912261963,
					["scale"] = 1,
					["t"] = 190.1714265346527,
					["open"] = true,
					["feature"] = "desktop_window",
					["name"] = "true_unitframes:targettarget_window",
					["alpha"] = 1,
					["anchor"] = "TOPLEFT",
					["l"] = 709.4857513298302,
					["b"] = 171.8857124447823,
					["dock"] = {
						["LEFT"] = {
							["id"] = "true_unitframes:target_window",
							["x"] = 5,
							["point"] = "RIGHT",
							["y"] = 0,
						},
					},
				},
			},
		};
	end
		-- Crowd Control Object
	if not core["cc_aurafilter"] then
		core["cc_aurafilter"] = {
			["ty"] = "AuraFilter",
			["version"] = 1,
			["data"] = {
				9484, -- [1]
				605, -- [2]
				8122, -- [3]
				64058, -- [4]
				91800, -- [5]
				5211, -- [6]
				33786, -- [7]
				339, -- [8]
				2637, -- [9]
				22570, -- [10]
				9005, -- [11]
				64803, -- [12]
				3355, -- [13]
				50245, -- [14]
				1513, -- [15]
				19503, -- [16]
				24394, -- [17]
				50518, -- [18]
				50519, -- [19]
				24131, -- [20]
				44572, -- [21]
				31661, -- [22]
				33395, -- [23]
				122, -- [24]
				68246, -- [25]
				64343, -- [26]
				118, -- [27]
				83073, -- [28]
				31589, -- [29]
				853, -- [30]
				2812, -- [31]
				20066, -- [32]
				20170, -- [33]
				10326, -- [34]
				2094, -- [35]
				1833, -- [36]
				1776, -- [37]
				408, -- [38]
				6770, -- [39]
				5211, -- [40]
				51514, -- [44]
				710, -- [45]
				6789, -- [46]
				5782, -- [47]
				5484, -- [48]
				6358, -- [50]
				30283, -- [51]
				53148, -- [52]
				12809, -- [53]
				20253, -- [54]
				1715, -- [55]
				23694, -- [56]
				20511, -- [57]
				46968, -- [58]
				88625, -- [59]
				87204, -- [60]
				116, -- [61]
				3409, -- [62]
				49203, -- [63]
			},
		};
	end
		-- Action Bar Objects
	if not core["actionbar_design"] then
		core["actionbar_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 60,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 360,
					["alpha"] = 1,
				}, -- [1]
				{
					["selfcast"] = 1,
					["externalButtonSkin"] = "true_bs_caith",
					["showtooltip"] = 1,
					["flo"] = 1,
					["rows"] = 1,
					["fontcount"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "RIGHT",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "BOTTOM",
						["size"] = 8,
					},
					["name"] = "defaultbarA",
					["feature"] = "actionbar",
					["iconspx"] = 0,
					["cd"] = {
						["HideText"] = 0,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["Offsetx"] = 0,
						["UpdateSpeed"] = 0.3,
						["TextType"] = "Seconds",
						["GfxReverse"] = true,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["iconspy"] = 0,
					["nIcons"] = 12,
					["showkey"] = 1,
					["barid"] = "mainbar1",
					["size"] = 30,
					["fontmacro"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["cr"] = 1,
						["flags"] = "OUTLINE",
						["cg"] = 1,
						["justifyH"] = "CENTER",
						["cb"] = 1,
						["ca"] = 0,
						["name"] = "Default",
						["title"] = "Default",
						["size"] = 8,
					},
					["flyoutdirection"] = "UP",
					["owner"] = "Base",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["headerstateCustom"] = "",
					["headerstateType"] = "Stance",
					["headervisiCustom"] = "[mod:alt] show; hide",
					["version"] = 1,
--[[				["bkd"] = {
						["_border"] = "fer4",
						["edgeSize"] = 8,
						["ba"] = 1,
						["_backdrop"] = "none",
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\Ferous\\Borders\\fer4",
						["bg"] = 0,
						["bb"] = 0,
						["br"] = 0,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["orientation"] = "RIGHT",
					["usebs"] = true,
					["headervisType"] = "None",
					["headervisiType"] = "None",
				}, -- [2]
				{
					["selfcast"] = 1,
					["flo"] = 1,
					["showtooltip"] = 1,
					["name"] = "defaultbarB",
					["rows"] = 1,
					["fontcount"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "RIGHT",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "BOTTOM",
						["size"] = 8,
					},
					["size"] = 30,
					["feature"] = "actionbar",
					["iconspx"] = 0,
					["cd"] = {
						["HideText"] = 0,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["Offsetx"] = 0,
						["UpdateSpeed"] = 0.3,
						["TextType"] = "Seconds",
						["GfxReverse"] = true,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = -30,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["iconspy"] = 0,
					["nIcons"] = 12,
					["showkey"] = 1,
					["barid"] = "bar6_druid_stealth",
					["usebs"] = true,
					["flyoutdirection"] = "UP",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["owner"] = "Base",
					["headerstateCustom"] = "",
--[[				["bkd"] = {
						["_border"] = "fer4",
						["edgeSize"] = 8,
						["ba"] = 1,
						["_backdrop"] = "none",
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\Ferous\\Borders\\fer4",
						["bg"] = 0,
						["bb"] = 0,
						["br"] = 0,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["headerstateType"] = "Actionbar",
					["headervisiCustom"] = "[mod:alt] show; hide",
					["version"] = 1,
					["fontmacro"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["cr"] = 1,
						["flags"] = "OUTLINE",
						["cg"] = 1,
						["justifyH"] = "CENTER",
						["cb"] = 1,
						["ca"] = 0,
						["name"] = "Default",
						["title"] = "Default",
						["size"] = 8,
					},
					["orientation"] = "RIGHT",
					["externalButtonSkin"] = "true_bs_caith",
					["headervisType"] = "None",
					["headervisiType"] = "None",
				}, -- [3]
			},
		};
	end
	if not core["actionbar_window"] then
		core["actionbar_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:actionbar_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["clickable"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
	if not core["sidebar_design"] then
		core["sidebar_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 360,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 90,
					["alpha"] = 1,
				}, -- [1]
				{
					["selfcast"] = 1,
					["flo"] = 1,
					["showtooltip"] = 1,
					["name"] = "sidebarA",
					["rows"] = 1,
					["fontcount"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "RIGHT",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "BOTTOM",
						["size"] = 8,
					},
					["size"] = 30,
					["feature"] = "actionbar",
					["iconspx"] = 0,
					["cd"] = {
						["GfxReverse"] = true,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["UpdateSpeed"] = 0.3,
						["Offsetx"] = 0,
						["TextType"] = "Seconds",
						["HideText"] = 0,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["iconspy"] = 0,
					["nIcons"] = 12,
					["showkey"] = 1,
					["barid"] = "bar3",
					["usebs"] = true,
					["flyoutdirection"] = "UP",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["owner"] = "Base",
					["headerstateCustom"] = "",
--[[				["bkd"] = {
						["_border"] = "fer4",
						["edgeSize"] = 8,
						["ba"] = 1,
						["bb"] = 0,
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\Ferous\\Borders\\fer4",
						["br"] = 0,
						["_backdrop"] = "none",
						["bg"] = 0,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["headerstateType"] = "Actionbar",
					["headervisiCustom"] = "[mod:alt] show; hide",
					["version"] = 1,
					["fontmacro"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
						["flags"] = "OUTLINE",
						["cg"] = 1,
						["justifyH"] = "CENTER",
						["cb"] = 1,
						["title"] = "Default",
						["name"] = "Default",
						["ca"] = 0,
						["cr"] = 1,
					},
					["orientation"] = "DOWN",
					["externalButtonSkin"] = "true_bs_caith",
					["headervisType"] = "None",
					["headervisiType"] = "None",
				}, -- [2]
				{
					["selfcast"] = 1,
					["flo"] = 1,
					["showtooltip"] = 1,
					["name"] = "sidebarB",
					["rows"] = 1,
					["fontcount"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "RIGHT",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "BOTTOM",
						["size"] = 8,
					},
					["size"] = 30,
					["feature"] = "actionbar",
					["iconspx"] = 0,
					["cd"] = {
						["GfxReverse"] = true,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["UpdateSpeed"] = 0.3,
						["Offsetx"] = 0,
						["TextType"] = "Seconds",
						["HideText"] = 0,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 30,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["iconspy"] = 0,
					["nIcons"] = 12,
					["showkey"] = 1,
					["barid"] = "bar4",
					["usebs"] = true,
					["flyoutdirection"] = "UP",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["owner"] = "Base",
					["headerstateCustom"] = "",
--[[				["bkd"] = {
						["_border"] = "fer4",
						["edgeSize"] = 8,
						["ba"] = 1,
						["bb"] = 0,
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\Ferous\\Borders\\fer4",
						["br"] = 0,
						["_backdrop"] = "none",
						["bg"] = 0,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["headerstateType"] = "Actionbar",
					["headervisiCustom"] = "[mod:alt] show; hide",
					["version"] = 1,
					["fontmacro"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
						["flags"] = "OUTLINE",
						["cg"] = 1,
						["justifyH"] = "CENTER",
						["cb"] = 1,
						["title"] = "Default",
						["name"] = "Default",
						["ca"] = 0,
						["cr"] = 1,
					},
					["orientation"] = "DOWN",
					["externalButtonSkin"] = "true_bs_caith",
					["headervisType"] = "None",
					["headervisiType"] = "None",
				}, -- [3]
				{
					["selfcast"] = 1,
					["externalButtonSkin"] = "true_bs_caith",
					["showtooltip"] = 1,
					["flo"] = 1,
					["rows"] = 1,
					["fontcount"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "RIGHT",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "BOTTOM",
						["size"] = 8,
					},
					["name"] = "sidebarC",
					["feature"] = "actionbar",
					["iconspx"] = 0,
					["cd"] = {
						["GfxReverse"] = true,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["UpdateSpeed"] = 0.3,
						["Offsetx"] = 0,
						["TextType"] = "Seconds",
						["HideText"] = 0,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 60,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["flags"] = "OUTLINE",
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["iconspy"] = 0,
					["nIcons"] = 12,
					["showkey"] = 1,
					["barid"] = "bar5",
					["size"] = 30,
					["fontmacro"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
						["flags"] = "OUTLINE",
						["cg"] = 1,
						["justifyH"] = "CENTER",
						["cb"] = 1,
						["title"] = "Default",
						["name"] = "Default",
						["ca"] = 0,
						["cr"] = 1,
					},
					["flyoutdirection"] = "UP",
					["owner"] = "Base",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["headerstateCustom"] = "",
					["headerstateType"] = "Actionbar",
					["headervisiCustom"] = "[mod:alt] show; hide",
					["version"] = 1,
--[[				["bkd"] = {
						["_border"] = "fer4",
						["edgeSize"] = 8,
						["ba"] = 1,
						["bb"] = 0,
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\Ferous\\Borders\\fer4",
						["br"] = 0,
						["_backdrop"] = "none",
						["bg"] = 0,
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["orientation"] = "DOWN",
					["usebs"] = true,
					["headervisType"] = "None",
					["headervisiType"] = "None",
				}, -- [4]
			},
		};
	end
	if not core["sidebar_window"] then
		core["sidebar_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:sidebar_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["clickable"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
		-- Core Design Objects 
	if not core["minimap_design"] then
		core["minimap_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 140,
					["version"] = 1,
					["w"] = 140,
					["alpha"] = 1,
				}, -- [1]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "decor",
					["w"] = "140",
					["feature"] = "texture",
					["h"] = "140",
					["version"] = 1,
					["name"] = "basebg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [2]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [3]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "basebg",
				}, -- [4]
				{
					["feature"] = "Subframe",
					["h"] = "138",
					["name"] = "buttonframe",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "138",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [5]
				{
					["blipType"] = "Blizzard",
					["maskType"] = "Square",
					["owner"] = "decor",
					["w"] = "138",
					["feature"] = "minimap",
					["h"] = "138",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["font"] = {
						["sr"] = 0,
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["sb"] = 0,
						["sa"] = 1,
						["sg"] = 0,
						["title"] = "Default",
						["sx"] = 1,
						["sy"] = -1,
						["justifyH"] = "CENTER",
						["name"] = "Default",
						["size"] = 9,
					},
					["name"] = "minimap",
				}, -- [6]
				{
					["feature"] = "minimapbutton",
					["h"] = "32",
					["name"] = "tracking",
					["version"] = 1,
					["mbuttontype"] = "MiniMapTracking",
					["owner"] = "buttonframe",
					["w"] = "32",
					["anchor"] = {
						["dx"] = 5,
						["dy"] = 5,
						["lp"] = "BOTTOMLEFT",
						["rp"] = "BOTTOMLEFT",
						["af"] = "buttonframe",
					},
				}, -- [7]
				{
					["feature"] = "minimapbutton",
					["h"] = "32",
					["version"] = 1,
					["mbuttontype"] = "MiniMapBattlefieldFrame",
					["anchor"] = {
						["dx"] = 40,
						["dy"] = 5,
						["lp"] = "BOTTOMLEFT",
						["rp"] = "BOTTOMLEFT",
						["af"] = "buttonframe",
					},
					["owner"] = "buttonframe",
					["w"] = "32",
					["name"] = "battlefield",
				}, -- [8]
				{
					["feature"] = "minimapbutton",
					["h"] = "32",
					["name"] = "instance",
					["anchor"] = {
						["dx"] = -5,
						["dy"] = -5,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "buttonframe",
					},
					["mbuttontype"] = "MiniMapLFGFrame",
					["owner"] = "buttonframe",
					["w"] = "32",
					["version"] = 1,
				}, -- [9]
				{
					["feature"] = "minimapbutton",
					["h"] = "32",
					["name"] = "mail",
					["mbuttontype"] = "MiniMapMailFrame",
					["anchor"] = {
						["dx"] = 5,
						["dy"] = -5,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "buttonframe",
					},
					["owner"] = "buttonframe",
					["w"] = "32",
					["version"] = 1,
				}, -- [10]
			},
		};
	end
	if not core["minimap_window"] then
		core["minimap_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:minimap_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["clickable"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
	if not core["chatframe_design"] then
		core["chatframe_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 135,
					["version"] = 1,
					["w"] = 350,
					["alpha"] = 1,
				}, -- [1]
				{
					["number"] = "1",
					["ts"] = "HH:MM STR",
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["owner"] = "decor",
					["w"] = "350",
					["feature"] = "chatframe",
					["h"] = "103",
					["name"] = "ChatFrame",
					["channel"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["version"] = 1,
					["font"] = {
						["sr"] = 0,
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\ARIALN.ttf",
						["justifyV"] = "CENTER",
						["sb"] = 0,
						["sa"] = 1,
						["sg"] = 0,
						["sy"] = -1,
						["sx"] = 1,
						["title"] = "Default",
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 10,
					},
				}, -- [2]
				{
					["hb"] = 1,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = -103,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["owner"] = "decor",
					["w"] = 350,
					["feature"] = "chatframeeditbox",
				}, -- [3]
			},
		};
	end
	if not core["chatframe_window"] then
		core["chatframe_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:chatframe_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["unit"] = "player",
				}, -- [3]
			},
		};
	end
	if not core["aura_buff_design"] then
		core["aura_buff_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 60,
					["version"] = 1,
					["w"] = 600,
					["alpha"] = 1,
				}, -- [1]
				{
					["separateown"] = "NONE",
					["point"] = "TOPRIGHT",
					["wrapafter"] = 20,
					["sortmethod"] = "INDEX",
					["feature"] = "sec_aura_icons",
					["cd"] = {
						["GfxReverse"] = true,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN&TEXT",
						["Offsety"] = 0,
						["UpdateSpeed"] = 0.300000011920929,
						["Offsetx"] = 0,
						["TextType"] = "Largest",
						["HideText"] = 0,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "Base",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = true,
					["template"] = "RDXAB30x30Template",
					["auraType"] = "BUFFS",
					["owner"] = "Base",
					["fontst"] = {
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\VFL\\Fonts\\LiberationSans-Regular.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
					["bsdefault"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["maxwraps"] = 1,
					["orientation"] = "LEFT",
					["xoffset"] = 0,
					["name"] = "buffs",
					["version"] = 1,
					["yoffset"] = 0,
				}, -- [2]
			},
		};
	end
	if not core["aura_buff_window"] then
		core["aura_buff_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:aura_buff_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
	if not core["aura_debuff_design"] then
		core["aura_debuff_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 60,
					["version"] = 1,
					["w"] = 600,
					["alpha"] = 1,
				}, -- [1]
				{
					["separateown"] = "NONE",
					["point"] = "TOPRIGHT",
					["wrapafter"] = 20,
					["sortmethod"] = "INDEX",
					["feature"] = "sec_aura_icons",
					["cd"] = {
						["HideText"] = 0,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN&TEXT",
						["Offsety"] = 0,
						["Offsetx"] = 0,
						["UpdateSpeed"] = 0.300000011920929,
						["TextType"] = "Largest",
						["GfxReverse"] = true,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "Base",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = true,
					["template"] = "RDXAB30x30Template",
					["auraType"] = "DEBUFFS",
					["owner"] = "Base",
					["fontst"] = {
						["name"] = "Default",
						["title"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\Addons\\VFL\\Fonts\\LiberationSans-Regular.ttf",
						["justifyV"] = "CENTER",
						["size"] = 8,
					},
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
					["bsdefault"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["maxwraps"] = 1,
					["orientation"] = "LEFT",
					["xoffset"] = 0,
					["name"] = "debuffs",
					["version"] = 1,
					["yoffset"] = 0,
				}, -- [2]
			},
		};
	end
	if not core["aura_debuff_window"] then
		core["aura_debuff_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:aura_debuff_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["version"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
	-- Core Pet Objects
	if not core["petbar_design"] then
		core["petbar_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 60,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 150,
					["alpha"] = 1,
				}, -- [1]
				{
					["showtooltip"] = 1,
					["rows"] = 5,
					["feature"] = "actionbarpet",
					["iconspx"] = 0,
					["cd"] = {
						["HideText"] = 0,
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["Offsetx"] = 0,
						["UpdateSpeed"] = 0.3,
						["TextType"] = "Seconds",
						["GfxReverse"] = false,
					},
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["fontkey"] = {
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "CENTER",
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["size"] = 12,
					},
					["iconspy"] = 0,
					["nIcons"] = 10,
					["showkey"] = 1,
					["flo"] = 5,
					["size"] = 30,
					["owner"] = "Base",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "actionbarpet",
					["abid"] = 1,
					["version"] = 1,
					["headervisiCustom"] = "",
					["orientation"] = "RIGHT",
					["usebs"] = true,
					["externalButtonSkin"] = "true_bs_caith",
					["headervisiType"] = "Pet",
				}, -- [2]
			},
		};
	end
	if not core["petbar_window"] then
		core["petbar_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_core:petbar_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["clickable"] = 1,
					["version"] = 1,
					["unit"] = "pet",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
			},
		};
	end
		-- Script Objects
	if not core["true_script_hp"] then
		core["true_script_hp"] = {
			["ty"] = "Script",
			["version"] = 1,
			["data"] = {
				["script"] = "text = \"\"\nif not unit:IsIncapacitated() then\n  if UnitIsDead(uid) then\n    text = \"Dead\";\n  elseif UnitIsGhost(uid) then\n    text = \"Ghost\";\n  elseif not UnitIsConnected(uid) then\n    text = \"Link Dead\";\n  else\n    text = VFL.Kay(unit:Health());\n  end\nelse\n  text = \"\";\nend",
			},
		};
	end
	if not core["true_script_hp_hppercent"] then
		core["true_script_hp_hppercent"] = {
			["ty"] = "Script",
			["version"] = 1,
			["data"] = {
				["script"] = "text = \"\"\nif not unit:IsIncapacitated() then\n  if UnitIsDead(uid) then\n    text = \"Dead\";\n  elseif UnitIsGhost(uid) then\n    text = \"Ghost\";\n  elseif not UnitIsConnected(uid) then\n    text = \"Link Dead\";\n  else\n    text = string.format(VFL.strtcolor(classColor) .. \"%0.0f\" .. VFL.strtcolor(green) .. \" %s\", unit:FracHealth()*100, VFL.Kay(unit:Health()));\n  end\nelse\n  text = \"\";\nend",
			},
		};
	end
	if not core["true_script_hpmissing"] then
		core["true_script_hpmissing"] = {
			["ty"] = "Script",
			["version"] = 1,
			["data"] = {
				["script"] = "text = \"\"\nif not unit:IsIncapacitated() then\n  if UnitIsDead(uid) then\n    text = \"Dead\";\n  elseif UnitIsGhost(uid) then\n    text = \"Ghost\";\n  elseif not UnitIsConnected(uid) then\n    text = \"Link Dead\";\n  else\n    text = string.format(\"-%s\", VFL.Kay(unit:MissingHealth()));\n  end\nelse\n  text = \"\";\nend",
			},
		};
	end
	if not core["true_script_fp"] then
		core["true_script_fp"] = {
			["ty"] = "Script",
			["version"] = 1,
			["data"] = {
				["script"] = "text = \"\"\ntext = VFL.Kay(unit:FracPower()*100);",
			},
		};
	end
	if not core["true_script_level_name"] then
		core["true_script_level_name"] = {
			["ty"] = "Script",
			["version"] = 1,
			["data"] = {
				["script"] = "text=\"\"\nlocal level = UnitLevel(uid)\nif level > 0 then\n  level = level\nelseif level < 0 then\n  level = VFL.strcolor(1,0,0)..\"??\"\nend\n\nlocal name = unit:GetProperName():sub(1, 15)\n\ntext = VFL.strtcolor(blue) .. level .. \" \" .. VFL.strtcolor(purple) .. name\n\n",
			},
		};
	end
		-- Backup Unitframe Objects
	if not core["true_design_template_alternativepower"] then
		core["true_design_template_alternativepower"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["h"] = "25",
					["name"] = "changename",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["version"] = 1,
					["secure"] = 1,
					["flo"] = 3,
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.7000000000000001,
						["b"] = 1,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.9,
						["g"] = 0,
						["b"] = 1,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [11]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [18]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["cameraType"] = "SetCameraZero",
					["name"] = "portrait",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [28]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
				}, -- [29]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [30]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [31]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{alternativepowerframe}",
				}, -- [32]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "alternativepowerframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [33]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "alternativepowerframe",
					["w"] = "244",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "alternativebg",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [34]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "alternativebg",
				}, -- [35]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [36]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [37]
				{
					["feature"] = "var_pred_health",
				}, -- [38]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [39]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["b"] = 0.3,
						["g"] = 0.3,
						["r"] = 0.3,
					},
					["raColor3"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0,
					},
				}, -- [40]
				{
					["feature"] = "var_castlag",
				}, -- [41]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [42]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [43]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [44]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [45]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
				}, -- [46]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "power",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
				}, -- [47]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [48]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["colorVar"] = "lightred",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "phealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["frac"] = "fh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["colorVar"] = "darkgrey",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "healthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["sublevel"] = 4,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [50]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [51]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [52]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [53]
				{
					["frac"] = "fp",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "powerframe",
					["w"] = "248",
					["colorVar"] = "classColor",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "powerbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [54]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "powerframe",
					["inverse"] = 1,
				}, -- [55]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [56]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [57]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [59]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [60]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["name"] = "casttext",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
				}, -- [61]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "casttimetext",
				}, -- [62]
				{
					["frac"] = "spell_fraclag",
					["sublevel"] = 3,
					["owner"] = "castframe",
					["w"] = "248",
					["colorVar"] = "red",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "lagbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [63]
				{
					["frac"] = "",
					["sublevel"] = 4,
					["owner"] = "castframe",
					["w"] = "248",
					["colorVar"] = "powerColor",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "castbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [64]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["texIcon"] = "",
					["TEIExpireState"] = "Hide",
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
				}, -- [65]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [66]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "alternativepowerframe",
				}, -- [67]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "242",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "alternativepowertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "alternativepowerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [68]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "alternativepowertexture",
				}, -- [69]
			},
		};
	end
	if not core["true_design_template_alternativecast"] then
		core["true_design_template_alternativecast"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["secure"] = 1,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["flo"] = 3,
					["h"] = "25",
					["name"] = "player",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.7000000000000001,
						["b"] = 1,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.9,
						["g"] = 0,
						["b"] = 1,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [11]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [18]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["cameraType"] = "SetCameraZero",
					["name"] = "portrait",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "powerbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [28]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [29]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "244",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "castbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [30]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [31]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [32]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [33]
				{
					["feature"] = "var_pred_health",
				}, -- [34]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [35]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["b"] = 0.3,
						["g"] = 0.3,
						["r"] = 0.3,
					},
					["raColor3"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0,
					},
				}, -- [36]
				{
					["feature"] = "var_castlag",
				}, -- [37]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [38]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [40]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [41]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
				}, -- [42]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "power",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
				}, -- [43]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [44]
				{
					["frac"] = "pfh",
					["sublevel"] = 3,
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [45]
				{
					["frac"] = "fh",
					["sublevel"] = 4,
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [46]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [47]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [48]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [49]
				{
					["frac"] = "fp",
					["sublevel"] = 3,
					["owner"] = "powerframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["version"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [50]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [51]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [52]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "242",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [53]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [54]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [55]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "casttext",
				}, -- [56]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\Addons\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "casttimetext",
				}, -- [57]
				{
					["frac"] = "spell_fraclag",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "castframe",
					["w"] = "242",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "red",
					["name"] = "lagbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["frac"] = "",
					["sublevel"] = 4,
					["owner"] = "castframe",
					["w"] = "242",
					["colorVar"] = "powerColor",
					["drawLayer"] = "BACKGROUND",
					["h"] = "23",
					["name"] = "castbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [59]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TEIExpireState"] = "Hide",
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [60]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [61]
			},
		};
	end


	-- true_unitframes Package
		-- Player Objects
	if not unitframes["player_design"] then
		unitframes["player_design"] = {
			["ty"] = "SymLink",
			["version"] = 3,
			["data"] = {
				["targetpath_5"] = "true_unitframes:player_design_default",
				["class"] = "class",
				["targetpath_7"] = "true_unitframes:player_design_default",
				["targetpath_8"] = "true_unitframes:player_design_default",
				["targetpath_2"] = "true_unitframes:player_design_default",
				["targetpath_3"] = "true_unitframes:player_design_paladin",
				["targetpath_4"] = "true_unitframes:player_design_default",
				["targetpath_1"] = "true_unitframes:player_design_default",
				["targetpath_10"] = "true_unitframes:player_design_deathknight",
				["targetpath_9"] = "true_unitframes:player_design_hunter",
				["targetpath_6"] = "true_unitframes:player_design_default",
			},
		};
	end
	if not unitframes["player_design_default"] then
		unitframes["player_design_default"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["flo"] = 3,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["h"] = "25",
					["secure"] = 1,
					["name"] = "player",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "lightred",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [10]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [11]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [12]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [13]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [14]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [15]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [16]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["name"] = "portrait",
					["cameraType"] = "SetCameraZero",
				}, -- [17]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [18]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [19]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [20]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [21]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [22]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [23]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [24]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [25]
				{
					["owner"] = "powerframe",
					["w"] = "25",
					["feature"] = "(True) Hunter Priority Icon",
					["drawLayer"] = "ARTWORK",
					["h"] = "25",
					["name"] = "priority",
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = -5,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "LEFT",
						["af"] = "powerframe",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = true,
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [26]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [27]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
				}, -- [28]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [29]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [30]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [31]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [32]
				{
					["feature"] = "var_pred_health",
				}, -- [33]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [34]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["b"] = 0.3,
						["g"] = 0.3,
						["r"] = 0.3,
					},
					["raColor3"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0,
					},
				}, -- [35]
				{
					["feature"] = "var_castlag",
				}, -- [36]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [37]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [38]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [39]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [40]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["h"] = "23",
				}, -- [41]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "power",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
				}, -- [42]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [43]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["frac"] = "fh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["sublevel"] = 4,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [45]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [48]
				{
					["frac"] = "fp",
					["sublevel"] = 3,
					["owner"] = "powerframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "powerframe",
					["inverse"] = 1,
				}, -- [50]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [51]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [52]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [53]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [54]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [55]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["name"] = "casttext",
					["h"] = "23",
				}, -- [56]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["name"] = "casttimetext",
					["h"] = "23",
				}, -- [57]
				{
					["frac"] = "spell_fraclag",
					["sublevel"] = 3,
					["owner"] = "castframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "red",
					["name"] = "lagbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["frac"] = "",
					["sublevel"] = 4,
					["owner"] = "castframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "powerColor",
					["name"] = "castbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [59]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["texIcon"] = "",
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["TEIExpireState"] = "Hide",
				}, -- [60]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [61]
			},
		};
	end
	if not unitframes["player_design_deathknight"] then
		unitframes["player_design_deathknight"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["h"] = "25",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["secure"] = 1,
					["flo"] = 3,
					["name"] = "player",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [9]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [10]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [11]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [12]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [13]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [14]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [15]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [16]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["name"] = "portrait",
					["cameraType"] = "SetCameraZero",
				}, -- [17]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [18]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [19]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [20]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [21]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [22]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [23]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [24]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [25]
				{
					["drawLayer"] = "ARTWORK",
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["w"] = "25",
					["ButtonSkinOffset"] = 5,
					["feature"] = "(True) Hunter Priority Icon",
					["h"] = "25",
					["name"] = "priority",
					["bkd"] = {
					},
					["anchor"] = {
						["dx"] = -5,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "LEFT",
						["af"] = "powerframe",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = 1,
					["owner"] = "powerframe",
				}, -- [26]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [27]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
				}, -- [28]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [29]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [30]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{alternativepowerframe}",
				}, -- [31]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "alternativepowerframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [32]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "alternativepowerframe",
					["w"] = "244",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "alternativebg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "alternativepowerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [33]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "alternativebg",
				}, -- [34]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [35]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [36]
				{
					["feature"] = "var_pred_health",
				}, -- [37]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [38]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["b"] = 0.3,
						["g"] = 0.3,
						["r"] = 0.3,
					},
					["raColor3"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0,
					},
				}, -- [39]
				{
					["feature"] = "var_castlag",
				}, -- [40]
				{
					["feature"] = "Vars: Rune Info",
					["TextureType"] = "Normal",
					["colors"] = {
						{
							["a"] = 1,
							["r"] = 1,
							["g"] = 0,
							["b"] = 0.2,
						}, -- [1]
						{
							["a"] = 1,
							["r"] = 0,
							["g"] = 1,
							["b"] = 0,
						}, -- [2]
						{
							["a"] = 1,
							["r"] = 0,
							["g"] = 0.6,
							["b"] = 1,
						}, -- [3]
						{
							["a"] = 1,
							["r"] = 0.6,
							["g"] = 0.3,
							["b"] = 0.6,
						}, -- [4]
					},
				}, -- [41]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [42]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [43]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [44]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [45]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["h"] = "23",
				}, -- [46]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "power",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
				}, -- [47]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [48]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["frac"] = "fh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["sublevel"] = 4,
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [50]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [51]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [52]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [53]
				{
					["frac"] = "fp",
					["sublevel"] = 3,
					["owner"] = "powerframe",
					["w"] = "248",
					["h"] = "23",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [54]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "powerframe",
					["inverse"] = 1,
				}, -- [55]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [56]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [57]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [59]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [60]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["name"] = "casttext",
					["h"] = "23",
				}, -- [61]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["name"] = "casttimetext",
					["h"] = "23",
				}, -- [62]
				{
					["frac"] = "spell_fraclag",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "castframe",
					["w"] = "248",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "red",
					["name"] = "lagbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [63]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "castframe",
					["w"] = "248",
					["h"] = "23",
					["sublevel"] = 4,
					["colorVar"] = "powerColor",
					["name"] = "castbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [64]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [65]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [66]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{runebgcolors}",
				}, -- [67]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune1bg",
					["colorVar1"] = "rune1_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [68]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune2bg",
					["colorVar1"] = "rune2_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [69]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune3bg",
					["colorVar1"] = "rune3_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [70]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune4bg",
					["colorVar1"] = "rune4_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [71]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune5bg",
					["colorVar1"] = "rune5_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [72]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "rune6bg",
					["colorVar1"] = "rune6_color",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [73]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "alternativepowerframe",
				}, -- [74]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune1",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "alternativepowerframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [75]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune2",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune1",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [76]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune3",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune2",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [77]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune4",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune3",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [78]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune5",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune4",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [79]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "rune6",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune5",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [80]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune1bg",
					["texture"] = "rune1",
				}, -- [81]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune2bg",
					["texture"] = "rune2",
				}, -- [82]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune3bg",
					["texture"] = "rune3",
				}, -- [83]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune4bg",
					["texture"] = "rune4",
				}, -- [84]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune5bg",
					["texture"] = "rune5",
				}, -- [85]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "rune6bg",
					["texture"] = "rune6",
				}, -- [86]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune1_color",
					["name"] = "rune1",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "alternativepowerframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [87]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune2_color",
					["name"] = "rune2",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune1",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [88]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune3_color",
					["name"] = "rune3",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune2",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [89]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune4_color",
					["name"] = "rune4",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune3",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [90]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune5_color",
					["name"] = "rune5",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune4",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [91]
				{
					["frac"] = "",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "alternativepowerframe",
					["w"] = "39.5",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "rune6_color",
					["name"] = "rune6",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_rune5",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [92]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune1",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune1",
					["texIcon"] = "",
				}, -- [93]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune2",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune2",
					["texIcon"] = "",
				}, -- [94]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune3",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune3",
					["texIcon"] = "",
				}, -- [95]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune4",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune4",
					["texIcon"] = "",
				}, -- [96]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune5",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune5",
					["texIcon"] = "",
				}, -- [97]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Full",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "VFL.Hundredths",
					["statusBar"] = "rune6",
					["text"] = "",
					["texIcon"] = "",
					["TEIExpireState"] = "Hide",
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "true",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "rune6",
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
				}, -- [98]
			},
		};
	end
	if not unitframes["player_design_paladin"] then
		unitframes["player_design_paladin"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["flo"] = 3,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["name"] = "player",
					["secure"] = 1,
					["h"] = "25",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.7500000000000001,
						["b"] = 0,
					},
					["name"] = "gold",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 0.5,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["name"] = "lightred",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [11]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [12]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [13]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "red",
					["feature"] = "ColorVariable: Static Color",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "alternativepowerbg",
					["colorVar1"] = "gold",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [18]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["name"] = "portrait",
					["cameraType"] = "SetCameraZero",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
					["owner"] = "main",
					["w"] = "248",
					["flOffset"] = 2,
				}, -- [21]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [27]
				{
					["owner"] = "powerframe",
					["w"] = "25",
					["feature"] = "(True) Hunter Priority Icon",
					["drawLayer"] = "ARTWORK",
					["h"] = "25",
					["name"] = "priority",
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = -5,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "LEFT",
						["af"] = "powerframe",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = true,
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [28]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [29]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 1,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
				}, -- [30]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [31]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [32]
				{
					["feature"] = "commentline",
					["name"] = "{alternativepowerframe}",
					["version"] = 1,
				}, -- [33]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "alternativepowerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [34]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "alternativepowerframe",
					["w"] = "244",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "alternativebg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "alternativepowerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [35]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "alternativebg",
				}, -- [36]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{ccframe}",
				}, -- [37]
				{
					["feature"] = "Subframe",
					["h"] = "25",
					["name"] = "ccframe",
					["flOffset"] = 1,
					["owner"] = "decor",
					["w"] = "250",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 6,
						["lp"] = "BOTTOMLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [38]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [39]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [40]
				{
					["editor"] = "function FracHealthWOG()\n  local heal = 2133+(0.209*GetSpellBonusHealing()+0.198*UnitAttackPower(uid));\n  local holypower\n  --  Word of Glory now has a 10 second cooldown\n  --  if UnitAura(uid, GetSpellInfo(90174)) then holypower = UnitPower(uid,9)+3;\n  if UnitAura(uid, GetSpellInfo(90174)) then holypower = 3;\n  else holypower = UnitPower(uid,9);\n  end\n  local a = (UnitHealth(uid)+(heal*holypower));\n  local b = UnitHealthMax(uid);\n  if(b<1) then return 0; end\n  a=a/b;\n  if a<0 then\n    return 0;\n  elseif a>1 then\n    return 1;\n  else\n    return a;\n  end\nend\nwog = FracHealthWOG(); \n\n \n",
					["feature"] = "var_scripted",
					["vartype"] = "FracVar",
					["name"] = "wog",
				}, -- [41]
				{
					["feature"] = "var_pred_health",
				}, -- [42]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [43]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor3"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.7000000000000001,
						["b"] = 1,
					},
					["filterNameList"] = {
					},
					["raColor4"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [44]
				{
					["feature"] = "var_castlag",
				}, -- [45]
				{
					["feature"] = "Variable: Number power",
					["powertype"] = "SPELL_POWER_HOLY_POWER",
					["name"] = "hp",
				}, -- [46]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [47]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [48]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [49]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [50]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["h"] = "23",
				}, -- [51]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "power",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
				}, -- [52]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [53]
				{
					["frac"] = "wog",
					["sublevel"] = 3,
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "gold",
					["name"] = "wogphealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [54]
				{
					["frac"] = "pfh",
					["sublevel"] = 4,
					["owner"] = "healthframe",
					["w"] = "248",
					["h"] = "23",
					["feature"] = "statusbar_horiz",
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [55]
				{
					["frac"] = "fh",
					["sublevel"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["colorVar"] = "darkgrey",
					["drawLayer"] = "ARTWORK",
					["h"] = "23",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [56]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [57]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [59]
				{
					["frac"] = "fp",
					["feature"] = "statusbar_horiz",
					["owner"] = "powerframe",
					["w"] = "248",
					["colorVar"] = "classColor",
					["sublevel"] = 3,
					["h"] = "23",
					["name"] = "powerbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [60]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "powerframe",
					["inverse"] = 1,
				}, -- [61]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [62]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [63]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "casttexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [64]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [65]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [66]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["name"] = "casttext",
					["h"] = "23",
				}, -- [67]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["name"] = "casttimetext",
					["h"] = "23",
				}, -- [68]
				{
					["frac"] = "spell_fraclag",
					["feature"] = "statusbar_horiz",
					["owner"] = "castframe",
					["w"] = "248",
					["h"] = "23",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "red",
					["name"] = "lagbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [69]
				{
					["frac"] = "",
					["sublevel"] = 4,
					["owner"] = "castframe",
					["w"] = "248",
					["colorVar"] = "powerColor",
					["drawLayer"] = "BACKGROUND",
					["h"] = "23",
					["name"] = "castbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [70]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["texIcon"] = "",
					["TEIExpireState"] = "Hide",
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
				}, -- [71]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [72]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [73]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "80",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "hp1",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "alternativepowerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [74]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "80",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "hp2",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_hp1",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [75]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "80",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "hp3",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_hp2",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [76]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "alternativepowerbg",
					["texture"] = "hp1",
				}, -- [77]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "alternativepowerbg",
					["texture"] = "hp2",
				}, -- [78]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "alternativepowerbg",
					["texture"] = "hp3",
				}, -- [79]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "hpiconframe",
					["flOffset"] = 1,
					["owner"] = "alternativepowerframe",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "alternativepowerframe",
					},
				}, -- [80]
				{
					["number"] = "hp",
					["rows"] = 1,
					["feature"] = "custom_icons",
					["iconspx"] = 1,
					["color4"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["ButtonSkinOffset"] = 0,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "hpiconframe",
					},
					["externalButtonSkin"] = "bs_default",
					["iconspy"] = 0,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
					["nIcons"] = 5,
					["drawLayer"] = "BACKGROUND",
					["h"] = 23,
					["bsdefault"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["owner"] = "hpiconframe",
					["w"] = 80,
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.7490000000000001,
						["b"] = 0,
					},
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.7490000000000001,
						["b"] = 0,
					},
					["version"] = 1,
					["name"] = "holypower",
					["color3"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.7490000000000001,
						["b"] = 0,
					},
					["orientation"] = "RIGHT",
					["usebs"] = true,
					["color5"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [81]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "ccframe",
				}, -- [82]
				{
					["disableClick"] = 1,
					["externalNameFilter"] = "true_core:cc_aurafilter",
					["filterName"] = 1,
					["filterNameList"] = {
					},
					["feature"] = "aura_icons",
					["iconspx"] = 1,
					["cd"] = {
						["Font"] = {
							["name"] = "Default",
							["title"] = "Default",
							["justifyH"] = "CENTER",
							["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
							["justifyV"] = "CENTER",
							["size"] = 10,
						},
						["TimerType"] = "COOLDOWN",
						["Offsety"] = 0,
						["Offsetx"] = 0,
						["UpdateSpeed"] = 0.3,
						["TextType"] = "Largest",
						["HideText"] = 0,
					},
					["ButtonSkinOffset"] = 0,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "ccframe",
					},
					["externalButtonSkin"] = "bs_default",
					["iconspy"] = 0,
					["nIcons"] = 10,
					["fontst"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "CENTER",
						["name"] = "Default",
						["size"] = 16,
					},
					["mindurationfilter"] = "",
					["auraType"] = "DEBUFFS",
					["rows"] = 1,
					["owner"] = "ccframe",
					["maxdurationfilter"] = "",
					["disableShowTooltip"] = 1,
					["bkd"] = {
						["_border"] = "PixelborderA",
						["edgeSize"] = 10,
						["_backdrop"] = "none",
						["edgeFile"] = "Interface\\Addons\\TrueAui\\media\\border",
						["insets"] = {
							["top"] = 1,
							["right"] = 1,
							["left"] = 1,
							["bottom"] = 1,
						},
					},
					["version"] = 1,
					["usebkd"] = 1,
					["name"] = "debuffs",
					["orientation"] = "RIGHT",
					["bsdefault"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["size"] = 24.1,
					["unitfilter"] = "",
				}, -- [83]
			},
		};
	end
	if not unitframes["player_design_hunter"] then
		unitframes["player_design_hunter"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["h"] = "25",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["secure"] = 1,
					["flo"] = 3,
					["name"] = "player",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "green",
					["color"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
				}, -- [8]
				{
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "lightred",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["energyColor"] = {
						["a"] = 1,
						["r"] = 0.75,
						["g"] = 0.75,
						["b"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.6500000000000001,
						["b"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.5,
						["b"] = 0.2,
					},
					["feature"] = "ColorVariable: Unit PowerType Color",
					["runeColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.75,
						["b"] = 1,
					},
					["rageColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [10]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [11]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [12]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "red",
					["feature"] = "ColorVariable: Static Color",
				}, -- [13]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [14]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [15]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [16]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "player",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["name"] = "portrait",
					["cameraType"] = "SetCameraZero",
				}, -- [17]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [18]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [19]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [20]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [21]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [22]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [23]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [24]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [25]
				{
					["owner"] = "powerframe",
					["bsdefault"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["w"] = "25",
--[[				["bkd"] = {
						["_border"] = "bdr_c",
						["edgeSize"] = 8,
						["edgeFile"] = "Interface\\AddOns\\RDX_mediapack\\sharedmedia\\borders\\bdr_c",
						["insets"] = {
							["top"] = 2,
							["right"] = 2,
							["left"] = 2,
							["bottom"] = 2,
						},
					},
]]
					["drawLayer"] = "ARTWORK",
					["h"] = "25",
					["name"] = "priority",
					["ButtonSkinOffset"] = 5,
					["anchor"] = {
						["dx"] = -5,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "LEFT",
						["af"] = "powerframe",
					},
					["externalButtonSkin"] = "true_bs_caith",
					["usebs"] = true,
					["feature"] = "(True) Hunter Priority Icon",
				}, -- [26]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [27]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "242",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [28]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "244",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [29]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [30]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [31]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [32]
				{
					["feature"] = "var_pred_health",
				}, -- [33]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [34]
				{
					["feature"] = "var_scripted",
					["name"] = "scshot",
					["power"] = 1,
					["vartype"] = "FracVar",
					["editor"] = "function FracFocusSCshot()\n  spell = UnitCastingInfo(uid);\n  local incomingfocus\n  if spell == GetSpellInfo(56641 or 77767) then incomingfocus = 9;\n  else incomingfocus = 0;\n  end\n  local a = (UnitPower(uid)+(incomingfocus));\n  local b = UnitPowerMax(uid);\n  if(b<1) then return 0; end\n  a=a/b;\n  if a<0 then\n    return 0;\n  elseif a>1 then\n    return 1;\n  else\n    return a;\n  end\nend\nscshot = FracFocusSCshot(); \n",
				}, -- [35]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor3"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["r"] = 0.3,
						["g"] = 0.3,
						["b"] = 0.3,
					},
				}, -- [36]
				{
					["feature"] = "var_castlag",
				}, -- [37]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [38]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "healthtexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [40]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [41]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "healthtext",
				}, -- [42]
				{
					["script"] = "true_core:true_script_fp",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "power",
				}, -- [43]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["sublevel"] = 3,
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["frac"] = "fh",
					["sublevel"] = 4,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["drawLayer"] = "ARTWORK",
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [45]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "61.25",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture1",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "61.25",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture2",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_powertexture1",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [48]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "61.25",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture3",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_powertexture2",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "61.25",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture4",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "RIGHT",
						["af"] = "Texture_powertexture3",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [50]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture1",
				}, -- [51]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture2",
				}, -- [52]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture3",
				}, -- [53]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture4",
				}, -- [54]
				{
					["frac"] = "scshot",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["sublevel"] = 3,
					["colorVar"] = "lightred",
					["name"] = "scshotbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["orientation"] = "HORIZONTAL",
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [55]
				{
					["frac"] = "fp",
					["sublevel"] = 4,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["version"] = 1,
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [56]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [57]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [58]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "242",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "casttexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [59]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [60]
				{
					["frac"] = "spell_fraclag",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "castframe",
					["w"] = "242",
					["sublevel"] = 3,
					["feature"] = "statusbar_horiz",
					["colorVar"] = "red",
					["name"] = "lagbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPRIGHT",
						["rp"] = "TOPRIGHT",
						["af"] = "castframe",
					},
					["orientation"] = "HORIZONTAL",
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [61]
				{
					["frac"] = "",
					["sublevel"] = 4,
					["owner"] = "castframe",
					["w"] = "242",
					["drawLayer"] = "BACKGROUND",
					["feature"] = "statusbar_horiz",
					["h"] = "23",
					["name"] = "castbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["version"] = 1,
					["colorVar"] = "powerColor",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [62]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [63]
			},
		};
	end
	if not unitframes["player_window"] then
		unitframes["player_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_unitframes:player_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["unitWatch"] = 1,
					["version"] = 1,
					["switchvehicle"] = 1,
					["clickable"] = 1,
					["unit"] = "player",
				}, -- [3]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "player",
					["version"] = 1,
					["mbFriendly"] = "true_unitframes:player_mousebindings",
				}, -- [4]
				{
					["interval"] = 0.03,
					["feature"] = "Event: Periodic Repaint",
					["slot"] = "RepaintAll",
				}, -- [5]
			},
		};
	end
	if not unitframes["player_mousebindings"] then
		unitframes["player_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["3"] = {
					["script"] = "true_unitframes:player_macro_exitvehicle",
					["action"] = "macro",
				},
				["2"] = {
					["action"] = "menu",
				},
			},
		};
	end
	if not unitframes["player_macro_exitvehicle"] then
		unitframes["player_macro_exitvehicle"] = {
			["ty"] = "Macro",
			["version"] = 1,
			["data"] = {
				["script"] = "/script VehicleExit();",
			},
		};
	end
		-- Target Objects
	if not unitframes["target_design"] then
		unitframes["target_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 250,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "250",
					["feature"] = "hotspot",
					["flo"] = 3,
					["name"] = "target",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["version"] = 1,
					["h"] = "25",
					["secure"] = 1,
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.75",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.7000000000000001,
						["r"] = 0,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0.9,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
				}, -- [11]
				{
					["energyColor"] = {
						["a"] = 1,
						["r"] = 0.75,
						["g"] = 0.75,
						["b"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.6500000000000001,
						["b"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.5,
						["b"] = 0.2,
					},
					["feature"] = "ColorVariable: Unit PowerType Color",
					["runeColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.75,
						["b"] = 1,
					},
					["rageColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{portraitframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "portraitframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["falseAlpha"] = 1,
					["version"] = 1,
					["flag"] = "true",
					["owner"] = "portraitframe",
					["trueAlpha"] = 0.07500000000000001,
					["feature"] = "shader_ca",
				}, -- [18]
				{
					["flOffset"] = 1,
					["owner"] = "portraitframe",
					["w"] = "248",
					["unit"] = "target",
					["feature"] = "portrait_3d2",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "portraitframe",
					},
					["cameraType"] = "SetCameraZero",
					["name"] = "portrait",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "248",
					["anchor"] = {
						["dx"] = -3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "powerbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [28]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
				}, -- [29]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "250",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [30]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [31]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [32]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [33]
				{
					["feature"] = "var_pred_health",
				}, -- [34]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [35]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor3"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["r"] = 0.3,
						["g"] = 0.3,
						["b"] = 0.3,
					},
				}, -- [36]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [37]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "healthtexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [38]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [39]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [40]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
				}, -- [41]
				{
					["script"] = "true_core:true_script_level_name",
					["color"] = "powerColor",
					["owner"] = "texthealthframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["name"] = "name",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
				}, -- [42]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "texthealthframe",
					["inverse"] = 1,
				}, -- [43]
				{
					["frac"] = "pfh",
					["sublevel"] = 3,
					["owner"] = "healthframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["drawLayer"] = "ARTWORK",
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["frac"] = "fh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "248",
					["sublevel"] = 4,
					["drawLayer"] = "ARTWORK",
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [45]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "powertexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [48]
				{
					["frac"] = "fp",
					["sublevel"] = 3,
					["owner"] = "powerframe",
					["w"] = "248",
					["feature"] = "statusbar_horiz",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["orientation"] = "HORIZONTAL",
					["h"] = "23",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["feature"] = "shader_showhide",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
					["owner"] = "powerframe",
					["inverse"] = 1,
				}, -- [50]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [51]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [52]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "248",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [53]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "casttextframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "248",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [54]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [55]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "casttextframe",
					},
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["name"] = "casttext",
				}, -- [56]
				{
					["script"] = "",
					["owner"] = "casttextframe",
					["w"] = "245",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "casttextframe",
					},
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
					["name"] = "casttimetext",
				}, -- [57]
				{
					["frac"] = "",
					["feature"] = "statusbar_horiz",
					["owner"] = "castframe",
					["w"] = "248",
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 3,
					["h"] = "23",
					["name"] = "castbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["version"] = 1,
					["colorVar"] = "powerColor",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [58]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "casttext",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "casttimetext",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "spell_name_rank",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [59]
				{
					["feature"] = "shader_showhide",
					["owner"] = "casttextframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [60]
			},
		};
	end
	if not unitframes["target_window"] then
		unitframes["target_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_unitframes:target_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["unitWatch"] = 1,
					["version"] = 1,
					["switchvehicle"] = 1,
					["clickable"] = 1,
					["unit"] = "target",
				}, -- [3]
				{
					["interval"] = 0.03,
					["feature"] = "Event: Periodic Repaint",
					["slot"] = "RepaintAll",
				}, -- [4]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "target",
					["version"] = 1,
					["mbFriendly"] = "true_unitframes:target_mousebindings",
				}, -- [5]
			},
		};
	end
	if not unitframes["target_mousebindings"] then
		unitframes["target_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["S3"] = {
					["action"] = "focus",
				},
				["2"] = {
					["action"] = "menu",
				},
			},
		};
	end
		-- Targettarget Objects
	if not unitframes["targettarget_design"] then
		unitframes["targettarget_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 150,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "150",
					["feature"] = "hotspot",
					["h"] = "25",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["secure"] = 1,
					["flo"] = 3,
					["name"] = "targettarget",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.7000000000000001,
						["r"] = 0,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0.9,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
				}, -- [11]
				{
					["energyColor"] = {
						["a"] = 1,
						["r"] = 0.75,
						["g"] = 0.75,
						["b"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.6500000000000001,
						["b"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.5,
						["b"] = 0.2,
					},
					["feature"] = "ColorVariable: Unit PowerType Color",
					["runeColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.75,
						["b"] = 1,
					},
					["rageColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "150",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [18]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "powerbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [24]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [25]
				{
					["feature"] = "var_pred_health",
				}, -- [26]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [28]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "148",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "healthtexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [29]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [30]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [31]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "healthtext",
				}, -- [32]
				{
					["color"] = "purple",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_np",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["trunc"] = 15,
					["name"] = "np",
					["h"] = "23",
				}, -- [33]
				{
					["frac"] = "pfh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["drawLayer"] = "ARTWORK",
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [34]
				{
					["frac"] = "fh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["drawLayer"] = "ARTWORK",
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["sublevel"] = 4,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [35]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [36]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [37]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [38]
				{
					["frac"] = "fp",
					["feature"] = "statusbar_horiz",
					["owner"] = "powerframe",
					["w"] = "142",
					["h"] = "23",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
			},
		};
	end
	if not unitframes["targettarget_window"] then
		unitframes["targettarget_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_unitframes:targettarget_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["unitWatch"] = 1,
					["switchvehicle"] = 1,
					["version"] = 1,
					["clickable"] = 1,
					["unit"] = "targettarget",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "targettarget",
					["version"] = 1,
					["mbFriendly"] = "true_unitframes:targettarget_mousebindings",
				}, -- [5]
			},
		};
	end
	if not unitframes["targettarget_mousebindings"] then
		unitframes["targettarget_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["S3"] = {
					["action"] = "focus",
				},
				["2"] = {
					["action"] = "menu",
				},
			},
		};
	end
		-- Focus Objects
	if not unitframes["focus_design"] then
		unitframes["focus_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 150,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "150",
					["feature"] = "hotspot",
					["secure"] = 1,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["flo"] = 3,
					["h"] = "25",
					["name"] = "focus",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.7000000000000001,
						["b"] = 1,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.9,
						["g"] = 0,
						["b"] = 1,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [11]
				{
					["feature"] = "ColorVariable: Unit PowerType Color",
					["rageColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["b"] = 0.2,
						["g"] = 0.5,
						["r"] = 1,
					},
					["energyColor"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0.75,
						["r"] = 0.75,
					},
					["runeColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.75,
						["r"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.6500000000000001,
						["r"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "150",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [18]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "powerbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "castbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [28]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [29]
				{
					["feature"] = "var_pred_health",
				}, -- [30]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [31]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["b"] = 0.3,
						["g"] = 0.3,
						["r"] = 0.3,
					},
					["raColor3"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0,
					},
				}, -- [32]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [33]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "148",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [34]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [35]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [36]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "RIGHT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["name"] = "healthtext",
					["h"] = "23",
				}, -- [37]
				{
					["color"] = "purple",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_np",
					["font"] = {
						["color"] = {
							["r"] = 1,
							["g"] = 1,
							["b"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["sy"] = 0,
						["sx"] = 0,
						["title"] = "Default",
						["name"] = "Default",
						["justifyH"] = "LEFT",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["trunc"] = 15,
					["h"] = "23",
					["name"] = "np",
				}, -- [38]
				{
					["frac"] = "pfh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
				{
					["frac"] = "fh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["sublevel"] = 4,
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [40]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [41]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "powertexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [42]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [43]
				{
					["frac"] = "fp",
					["feature"] = "statusbar_horiz",
					["owner"] = "powerframe",
					["w"] = "142",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [45]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "casttexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [48]
				{
					["frac"] = "",
					["feature"] = "statusbar_horiz",
					["owner"] = "castframe",
					["w"] = "142",
					["colorVar"] = "powerColor",
					["drawLayer"] = "BACKGROUND",
					["h"] = "23",
					["name"] = "castbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["version"] = 1,
					["sublevel"] = 3,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "",
					["texIcon"] = "",
					["color1"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "spell",
					["TEIExpireState"] = "Hide",
				}, -- [50]
			},
		};
	end
	if not unitframes["focus_window"] then
		unitframes["focus_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_unitframes:focus_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["unitWatch"] = 1,
					["version"] = 1,
					["switchvehicle"] = 1,
					["clickable"] = 1,
					["unit"] = "focus",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "focus",
					["version"] = 1,
					["mbFriendly"] = "true_unitframes:focus_mousebindings",
				}, -- [5]
			},
		};
	end
	if not unitframes["focus_mousebindings"] then
		unitframes["focus_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["S3"] = {
					["action"] = "focus",
				},
				["2"] = {
					["action"] = "menu",
				},
			},
		};
	end
		-- Pet Objects
	if not unitframes["pet_design"] then
		unitframes["pet_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 25,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 150,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "150",
					["feature"] = "hotspot",
					["secure"] = 1,
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["flo"] = 3,
					["h"] = "25",
					["name"] = "pet",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.7000000000000001,
						["r"] = 0,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0.9,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
				}, -- [11]
				{
					["energyColor"] = {
						["a"] = 1,
						["r"] = 0.75,
						["g"] = 0.75,
						["b"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.6500000000000001,
						["b"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.5,
						["b"] = 0.2,
					},
					["feature"] = "ColorVariable: Unit PowerType Color",
					["runeColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.75,
						["b"] = 1,
					},
					["rageColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "150",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "healthbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [18]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "castbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [28]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [29]
				{
					["feature"] = "var_pred_health",
				}, -- [30]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [31]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor3"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["r"] = 0.3,
						["g"] = 0.3,
						["b"] = 0.3,
					},
				}, -- [32]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [33]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "148",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "healthtexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [34]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [35]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [36]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_custom",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["h"] = "23",
					["name"] = "healthtext",
				}, -- [37]
				{
					["color"] = "purple",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_np",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["trunc"] = 15,
					["name"] = "np",
					["h"] = "23",
				}, -- [38]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "lightred",
					["name"] = "phealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
				{
					["frac"] = "fh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "148",
					["h"] = "23",
					["sublevel"] = 4,
					["colorVar"] = "darkgrey",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [40]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [41]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [42]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [43]
				{
					["frac"] = "fp",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "powerframe",
					["w"] = "142",
					["h"] = "23",
					["sublevel"] = 3,
					["colorVar"] = "classColor",
					["name"] = "powerbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [45]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [48]
				{
					["frac"] = "",
					["sublevel"] = 3,
					["owner"] = "castframe",
					["w"] = "142",
					["colorVar"] = "powerColor",
					["drawLayer"] = "BACKGROUND",
					["h"] = "23",
					["name"] = "castbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [50]
			},
		};
	end
	if not unitframes["pet_window"] then
		unitframes["pet_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_unitframes:focus_design",
				}, -- [2]
				{
					["feature"] = "layout_single_unitframe",
					["unitWatch"] = 1,
					["switchvehicle"] = 1,
					["version"] = 1,
					["clickable"] = 1,
					["unit"] = "pet",
				}, -- [3]
				{
					["feature"] = "Event: Periodic Repaint",
					["interval"] = 0.03,
					["slot"] = "RepaintAll",
				}, -- [4]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "pet",
					["version"] = 1,
					["mbFriendly"] = "true_unitframes:pet_mousebindings",
				}, -- [5]
			},
		};
	end
	if not unitframes["pet_mousebindings"] then
		unitframes["pet_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["S3"] = {
					["action"] = "focus",
				},
				["2"] = {
					["action"] = "menu",
				},
			},
		};
	end

	-- true_multiframes Package
		-- Raid Objects
	if not multiframes["raid_design"] then
		multiframes["raid_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 28.7,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 43.7,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "40",
					["feature"] = "hotspot",
					["secure"] = 1,
					["name"] = "raid",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["version"] = 1,
					["flo"] = 3,
					["h"] = "25",
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 1,
					["owner"] = "decor",
					["w"] = "38",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "darkgrey",
					["color"] = {
						["a"] = 1,
						["r"] = 0.15,
						["g"] = 0.15,
						["b"] = 0.15,
					},
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0.9,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 1,
						["r"] = 0.25,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0.5,
					},
				}, -- [11]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [12]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "38",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [13]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "40",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [14]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [16]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [17]
				{
					["feature"] = "var_pred_health",
				}, -- [18]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [19]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "38",
					["feature"] = "texture",
					["h"] = "23",
					["name"] = "healthtexture",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["sublevel"] = 2,
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [20]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [21]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "38",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [22]
				{
					["color"] = "classColor",
					["owner"] = "texthealthframe",
					["w"] = "38",
					["feature"] = "txt_np",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "CENTER",
						["name"] = "Default",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 2,
						["dy"] = 5,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "texthealthframe",
					},
					["trunc"] = 4,
					["name"] = "np",
					["h"] = "23",
				}, -- [23]
				{
					["script"] = "true_core:true_script_hpmissing",
					["color"] = "green",
					["owner"] = "texthealthframe",
					["w"] = "38",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 2,
						["dy"] = -5,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "texthealthframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "CENTER",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "healthtext",
				}, -- [24]
				{
					["frac"] = "pfh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "38",
					["colorVar"] = "red",
					["sublevel"] = 3,
					["h"] = "23",
					["name"] = "phealthbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [25]
				{
					["frac"] = "fh",
					["feature"] = "statusbar_horiz",
					["owner"] = "healthframe",
					["w"] = "38",
					["colorVar"] = "darkgrey",
					["sublevel"] = 4,
					["h"] = "23",
					["name"] = "healthbar",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["drawLayer"] = "ARTWORK",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
			},
		};
	end
	if not multiframes["raid_window"] then
		multiframes["raid_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_multiframes:raid_design",
				}, -- [2]
				{
					["feature"] = "Data Source: Secure Set",
					["rostertype"] = "RAID",
					["set"] = {
						["class"] = "file",
						["file"] = "true_multiframes:raid_filterset",
					},
				}, -- [3]
				{
					["bkt"] = 2,
					["switchvehicle"] = 1,
					["dxn"] = 1,
					["cols"] = 5,
					["feature"] = "Header Grid",
					["axis"] = 1,
				}, -- [4]
				{
					["feature"] = "FilterSet Editor",
				}, -- [5]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "raid",
					["version"] = 1,
					["mbFriendly"] = "true_multiframes:raid_mousebindings",
				}, -- [6]
				{
					["feature"] = "mp_args",
					["period"] = 0.01,
					["version"] = 1,
					["dpm1"] = 0,
				}, -- [7]
			},
		};
	end
	if not multiframes["raid_filterset"] then
		multiframes["raid_filterset"] = {
			["ty"] = "FilterSet",
			["version"] = 1,
			["data"] = {
				"and", -- [1]
				{
					"groups", -- [1]
					true, -- [2]
					true, -- [3]
					true, -- [4]
					true, -- [5]
					true, -- [6]
					true, -- [7]
					true, -- [8]
					true, -- [9]
				}, -- [2]
				{
					"classes", -- [1]
					true, -- [2]
					true, -- [3]
					true, -- [4]
					true, -- [5]
					true, -- [6]
					true, -- [7]
					true, -- [8]
					true, -- [9]
					true, -- [10]
					true, -- [11]
					true, -- [12]
				}, -- [3]
			},
		};
	end
	if not multiframes["raid_mousebindings"] then
		multiframes["raid_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["3"] = {
					["action"] = "cast",
					["spell"] = 1022,
				},
				["2"] = {
					["action"] = "cast",
					["spell"] = 85673,
				},
				["5"] = {
					["action"] = "cast",
					["spell"] = "Cleanse",
				},
				["S1"] = {
					["action"] = "cast",
					["spell"] = 1044,
				},
				["S2"] = {
					["action"] = "cast",
					["spell"] = 6940,
				},
			},
		};
	end
		-- Party Objects
	if not multiframes["party_design"] then
		multiframes["party_design"] = {
			["ty"] = "Design",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "base_default",
					["h"] = 50,
					["version"] = 1,
					["hlt"] = 1,
					["w"] = 150,
					["alpha"] = 1,
				}, -- [1]
				{
					["mover"] = 1,
					["w"] = "150",
					["feature"] = "hotspot",
					["h"] = "25",
					["name"] = "party",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "Base",
					},
					["flo"] = 3,
					["secure"] = 1,
					["version"] = 1,
				}, -- [2]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "main",
					["flOffset"] = 2,
					["owner"] = "decor",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 1,
						["dy"] = -1,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "Base",
					},
				}, -- [3]
				{
					["feature"] = "commentline",
					["name"] = "color",
					["version"] = 1,
				}, -- [4]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0,
						["g"] = 0,
						["r"] = 0,
					},
					["name"] = "black",
					["feature"] = "ColorVariable: Static Color",
				}, -- [5]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 0.15,
						["g"] = 0.15,
						["r"] = 0.15,
					},
					["name"] = "darkgrey",
					["feature"] = "ColorVariable: Static Color",
				}, -- [6]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "healthbg",
					["colorVar1"] = "darkgrey",
					["colorVar2"] = "black",
					["bfVar"] = "0.5",
				}, -- [7]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0.7000000000000001,
						["r"] = 0,
					},
					["name"] = "blue",
					["feature"] = "ColorVariable: Static Color",
				}, -- [8]
				{
					["color"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 0,
						["r"] = 0.9,
					},
					["name"] = "purple",
					["feature"] = "ColorVariable: Static Color",
				}, -- [9]
				{
					["color"] = {
						["a"] = 1,
						["r"] = 0.25,
						["g"] = 1,
						["b"] = 0,
					},
					["name"] = "green",
					["feature"] = "ColorVariable: Static Color",
				}, -- [10]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "lightred",
					["color"] = {
						["a"] = 0.5,
						["b"] = 0,
						["g"] = 0,
						["r"] = 1,
					},
				}, -- [11]
				{
					["energyColor"] = {
						["a"] = 1,
						["r"] = 0.75,
						["g"] = 0.75,
						["b"] = 0,
					},
					["manaColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.6500000000000001,
						["b"] = 1,
					},
					["focusColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0.5,
						["b"] = 0.2,
					},
					["feature"] = "ColorVariable: Unit PowerType Color",
					["runeColor"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0.75,
						["b"] = 1,
					},
					["rageColor"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [12]
				{
					["feature"] = "ColorVariable: Unit Class Color",
				}, -- [13]
				{
					["feature"] = "ColorVariable: Two-Color Blend",
					["name"] = "powerbg",
					["colorVar1"] = "classColor",
					["colorVar2"] = "black",
					["bfVar"] = "0.6",
				}, -- [14]
				{
					["feature"] = "ColorVariable: Static Color",
					["name"] = "red",
					["color"] = {
						["a"] = 1,
						["r"] = 0.5,
						["g"] = 0,
						["b"] = 0,
					},
				}, -- [15]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{healthframe}",
				}, -- [16]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "healthframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [17]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "150",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "healthbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [18]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "healthbg",
				}, -- [19]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{powerframe}",
				}, -- [20]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "powerframe",
					["flOffset"] = 1,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = 3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [21]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "powerframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["version"] = 1,
					["name"] = "powerbg",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "powerframe",
					},
					["sublevel"] = 1,
					["drawLayer"] = "BACKGROUND",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [22]
				{
					["flag"] = "true",
					["feature"] = "Highlight: Texture Map",
					["color"] = "black",
					["texture"] = "powerbg",
				}, -- [23]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "{castframe}",
				}, -- [24]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "castframe",
					["flOffset"] = 2,
					["owner"] = "main",
					["w"] = "142",
					["anchor"] = {
						["dx"] = 3,
						["dy"] = -3,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "main",
					},
				}, -- [25]
				{
					["cleanupPolicy"] = 3,
					["owner"] = "castframe",
					["w"] = "144",
					["feature"] = "texture",
					["h"] = "25",
					["name"] = "castbg",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 1,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [26]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "black",
					["texture"] = "castbg",
				}, -- [27]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "variables",
				}, -- [28]
				{
					["feature"] = "Variable: Fractional health (fh)",
				}, -- [29]
				{
					["feature"] = "var_pred_health",
				}, -- [30]
				{
					["feature"] = "Variable: Fractional mana (fm)",
					["powertype"] = "nil",
					["name"] = "fp",
				}, -- [31]
				{
					["feature"] = "var_spellinfo",
					["raColor2"] = {
						["a"] = 1,
						["r"] = 1,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor1"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 1,
						["b"] = 0,
					},
					["raColor3"] = {
						["a"] = 1,
						["r"] = 0,
						["g"] = 0,
						["b"] = 1,
					},
					["raColor4"] = {
						["a"] = 1,
						["r"] = 0.3,
						["g"] = 0.3,
						["b"] = 0.3,
					},
				}, -- [32]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "healthframe",
				}, -- [33]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "healthframe",
					["w"] = "148",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "healthtexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["drawLayer"] = "ARTWORK",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [34]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "healthbg",
					["texture"] = "healthtexture",
				}, -- [35]
				{
					["feature"] = "Subframe",
					["h"] = "23",
					["name"] = "texthealthframe",
					["flOffset"] = 5,
					["owner"] = "healthframe",
					["w"] = "148",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "CENTER",
						["rp"] = "CENTER",
						["af"] = "healthframe",
					},
				}, -- [36]
				{
					["script"] = "true_core:true_script_hp_hppercent",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_custom",
					["h"] = "23",
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "LEFT",
						["rp"] = "LEFT",
						["af"] = "texthealthframe",
					},
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "RIGHT",
						["name"] = "Default",
						["size"] = 8,
					},
					["name"] = "healthtext",
				}, -- [37]
				{
					["color"] = "purple",
					["owner"] = "texthealthframe",
					["w"] = "145",
					["feature"] = "txt_np",
					["font"] = {
						["color"] = {
							["b"] = 1,
							["g"] = 1,
							["r"] = 1,
						},
						["face"] = "Interface\\AddOns\\TrueAui\\media\\pf_arma_five.ttf",
						["justifyV"] = "CENTER",
						["flags"] = "OUTLINE",
						["title"] = "Default",
						["sx"] = 0,
						["sy"] = 0,
						["justifyH"] = "LEFT",
						["name"] = "Default",
						["size"] = 8,
					},
					["version"] = 1,
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "RIGHT",
						["rp"] = "RIGHT",
						["af"] = "texthealthframe",
					},
					["trunc"] = 15,
					["name"] = "np",
					["h"] = "23",
				}, -- [38]
				{
					["frac"] = "pfh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "148",
					["colorVar"] = "lightred",
					["sublevel"] = 3,
					["h"] = "23",
					["name"] = "phealthbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [39]
				{
					["frac"] = "fh",
					["drawLayer"] = "ARTWORK",
					["owner"] = "healthframe",
					["w"] = "148",
					["colorVar"] = "darkgrey",
					["sublevel"] = 4,
					["h"] = "23",
					["name"] = "healthbar",
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "healthframe",
					},
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [40]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "powerframe",
				}, -- [41]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "powerframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "powertexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [42]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "powertexture",
				}, -- [43]
				{
					["frac"] = "fp",
					["drawLayer"] = "BACKGROUND",
					["owner"] = "powerframe",
					["w"] = "142",
					["colorVar"] = "classColor",
					["sublevel"] = 3,
					["h"] = "23",
					["name"] = "powerbar",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "powerframe",
					},
					["orientation"] = "HORIZONTAL",
					["version"] = 1,
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [44]
				{
					["feature"] = "commentline",
					["version"] = 1,
					["name"] = "castframe",
				}, -- [45]
				{
					["feature"] = "shader_showhide",
					["owner"] = "castframe",
					["version"] = 1,
					["flag"] = "spell_castingOrChanneled",
				}, -- [46]
				{
					["cleanupPolicy"] = 1,
					["owner"] = "castframe",
					["w"] = "142",
					["feature"] = "texture",
					["h"] = "23",
					["version"] = 1,
					["name"] = "casttexture",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["drawLayer"] = "BACKGROUND",
					["sublevel"] = 2,
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [47]
				{
					["feature"] = "Highlight: Texture Map",
					["flag"] = "true",
					["color"] = "powerbg",
					["texture"] = "casttexture",
				}, -- [48]
				{
					["frac"] = "",
					["sublevel"] = 3,
					["owner"] = "castframe",
					["w"] = "142",
					["h"] = "23",
					["drawLayer"] = "BACKGROUND",
					["colorVar"] = "powerColor",
					["name"] = "castbar",
					["version"] = 1,
					["orientation"] = "HORIZONTAL",
					["anchor"] = {
						["dx"] = 0,
						["dy"] = 0,
						["lp"] = "TOPLEFT",
						["rp"] = "TOPLEFT",
						["af"] = "castframe",
					},
					["feature"] = "statusbar_horiz",
					["texture"] = {
						["blendMode"] = "BLEND",
						["path"] = "Interface\\AddOns\\TrueAui\\media\\fer25",
					},
				}, -- [49]
				{
					["TEIDefaultState"] = "Hide",
					["textInfo"] = "",
					["sbExpireState"] = "Hide",
					["TTIExpireState"] = "Hide",
					["feature"] = "free_timer",
					["sbDefaultState"] = "Hide",
					["TTIDefaultState"] = "Hide",
					["textType"] = "Hundredths",
					["statusBar"] = "castbar",
					["text"] = "",
					["TEIExpireState"] = "Hide",
					["color1"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["color2"] = {
						["a"] = 1,
						["b"] = 1,
						["g"] = 1,
						["r"] = 1,
					},
					["TL"] = 0,
					["version"] = 1,
					["countTypeFlag"] = "spell_castingOrChanneled",
					["tex"] = "",
					["txt"] = "",
					["timerVar"] = "spell",
					["texIcon"] = "",
				}, -- [50]
			},
		};
	end
	if not multiframes["party_window"] then
		multiframes["party_window"] = {
			["ty"] = "Window",
			["version"] = 1,
			["data"] = {
				{
					["feature"] = "Frame: None",
					["bkd"] = {
						["_border"] = "none",
						["_backdrop"] = "none",
					},
				}, -- [1]
				{
					["feature"] = "Design",
					["design"] = "true_multiframes:party_design",
				}, -- [2]
				{
					["feature"] = "Data Source: Secure Set",
					["set"] = {
						["file"] = "true_multiframes:party_filterset",
						["class"] = "file",
					},
					["rostertype"] = "RAID",
				}, -- [3]
				{
					["bkt"] = 2,
					["switchvehicle"] = 1,
					["dxn"] = 1,
					["cols"] = 5,
					["feature"] = "Header Grid",
					["axis"] = 1,
				}, -- [4]
				{
					["feature"] = "FilterSet Editor",
				}, -- [5]
				{
					["feature"] = "mousebindings",
					["hotspot"] = "party",
					["version"] = 1,
					["mbFriendly"] = "true_multiframes:party_mousebindings",
				}, -- [6]
				{
					["feature"] = "mp_args",
					["version"] = 1,
					["dpm1"] = 0,
					["period"] = 0.01,
				}, -- [7]
			},
		};
	end
	if not multiframes["party_filterset"] then
		multiframes["party_filterset"] = {
			["ty"] = "FilterSet",
			["version"] = 1,
			["data"] = {
				"and", -- [1]
				{
					"mygroup", -- [1]
				}, -- [2]
				{
					"classes", -- [1]
					true, -- [2]
					true, -- [3]
					true, -- [4]
					true, -- [5]
					true, -- [6]
					true, -- [7]
					true, -- [8]
					true, -- [9]
					true, -- [10]
					true, -- [11]
					true, -- [12]
				}, -- [3]
				{
					"not", -- [1]
					{
						"me", -- [1]
					}, -- [2]
				}, -- [4]
			},
		};
	end
	if not multiframes["party_mousebindings"] then
		multiframes["party_mousebindings"] = {
			["ty"] = "MouseBindings",
			["version"] = 1,
			["data"] = {
				["1"] = {
					["action"] = "target",
				},
				["3"] = {
					["action"] = "cast",
					["spell"] = 1022,
				},
				["2"] = {
					["action"] = "cast",
					["spell"] = 85673,
				},
				["5"] = {
					["action"] = "cast",
					["spell"] = "Cleanse",
				},
				["S1"] = {
					["action"] = "cast",
					["spell"] = 1044,
				},
				["S2"] = {
					["action"] = "cast",
					["spell"] = 6940,
				},
			},
		};
	end
end);